/*

    Requires the Qt widget libraries, available at no cost at
    http://www.troll.no

    Copyright (C) 1997 Peter Putzer
                       pputzer@edu.uni-klu.ac.at

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "typelayout.h"
#include "copydialog.h"
#include <kapp.h>

// include meta-object code
#include "copydialog.moc"

KSVCopyDialog::KSVCopyDialog(QWidget *parent=0, const char *name=0) : QDialog(parent,name,TRUE) {
  TLForm* f = new TLForm("root", name, this, this->parentWidget());
  KTypeLayout* l = f->layout;

  l->addLabel("description", QString().sprintf(i18n("%s to:"), i18n(name)).data());
  l->newLine();

  l->addGroup("runlevel", i18n("Runlevel"), TRUE);
  l->newLine();
  for (int i = 0; i < 7; ++i) {
    rlBt[i] = (QRadioButton*) l->addRadioButton(QString().sprintf("button %i", i).data(), "runlevel", QString().sprintf("Runlevel &%i", i).data());
    
    if ( i%2 == 0)
      l->skip();
    
    if ( i%2 == 1 ) 
      l->newLine();
    
  }

  rlBt[0]->setChecked(TRUE);

  // set alignment
  //  l->setAlign("start", AlignLeft|AlignVCenter);
  //  l->setAlign("stop", AlignLeft|AlignVCenter);

  l->endGroup(); // runlevel

  l->newLine();

  l->addGroup("action", i18n("Action"), TRUE);
  l->newLine();
  startBt = (QRadioButton*) l->addRadioButton("start", "action", i18n("&Start"), TRUE, 0)->widget;
  l->newLine();
  stopBt = (QRadioButton*) l->addRadioButton("stop", "action", i18n("St&op"), FALSE, 1)->widget;

  // set alignment
  l->setAlign("start", AlignLeft|AlignVCenter);
  l->setAlign("stop", AlignLeft|AlignVCenter);

  l->endGroup(); // action
  l->newLine();

  l->addGroup ("buttons");
  l->skip();
  okButton = (QPushButton*) l->addButton("ok",i18n("OK"))->widget;
  cancelButton = (QPushButton*) l->addButton("cancel",i18n("Cancel"))->widget;
  l->setAlign("ok",AlignRight|AlignVCenter);
  l->setAlign("cancel",AlignVCenter);
  l->endGroup(); // buttons
 
  //  l->endGroup();
  l->activate();

  // setup some stuff
  okButton->setAutoDefault(TRUE);
  okButton->setDefault(FALSE);
  cancelButton->setAutoDefault(TRUE);
  cancelButton->setDefault(TRUE);
  cancelButton->setFocus();

  connect(okButton, SIGNAL(clicked()), this, SLOT(slotAccept()));
  connect(cancelButton, SIGNAL(clicked()), this, SLOT(slotReject()));

  // set fixed size
  setFixedSize(size());
}

KSVCopyDialog::~KSVCopyDialog() {

}

void KSVCopyDialog::slotAccept() {
  int val = rlBox->getValue();

  if ((val >= 0) && (val <= 6)) {
    accept();
  }
}

void KSVCopyDialog::slotReject() {
  reject();
}
