/*

    Requires the Qt widget libraries, available at no cost at
    http://www.troll.no

    Copyright (C) 1997 Peter Putzer
                       pputzer@edu.uni-klu.ac.at

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/     

#include <qpainter.h>
#include <qkeycode.h>
#include <qtooltip.h>
#include "draglist.h"

#include <stdlib.h>
#include <kapp.h>

// include meta-object code
#include "draglist.moc"

// define constants
#define ENTRY_HEIGHT 19


KSVDragList::KSVDragList(QWidget* parent = 0, const char* name = 0)
: KDNDWidget(parent, name){
  kil = new KIconLoader();
  dz = new KDNDDropZone(this, DndText);
  
  // connect the dropZone with my dropAction
  connect(dz, SIGNAL(dropAction(KDNDDropZone* )), this, SLOT(slotDropEvent( KDNDDropZone* )));
  // set background mode
  setBackgroundMode( NoBackground );

  // init (double-) buffer
  buffer = QPixmap();
  
  adjustSize();

  // initialize other class members
  focus = 0;
  oldFocus = -1;
  pressed = FALSE;
  removable = TRUE;
  runlevel = "Foo";
  origin = FALSE;
}

KSVDragList::~KSVDragList() {
  delete kil;
}

void KSVDragList::dndMouseMoveEvent ( QMouseEvent* _mouse ) {
  // 'pressed' is set in mousePressedEvent(...)
  if ( !pressed )
    return;
  
  int x = _mouse->pos().x();
  int y = _mouse->pos().y();
  
  if ( abs( x - press_x ) > Dnd_X_Precision || abs( y - press_y ) > Dnd_Y_Precision ) {
  //   bool foo; // not needed.. only for compatibility
    KSVDragData* localO = getObject(y);
    QString data = localO->toString();

    QPoint p = mapToGlobal( _mouse->pos() );
    QPixmap pixmap = localO->drawIcon();

    int dx = - pixmap.width() / 2;
    int dy = - pixmap.height() / 2;
    
    startDrag( new KDNDIcon( pixmap, p.x() + dx, p.y() + dy ), data.data(), data.length(), DndText, dx, dy );
    pressed = FALSE;
  } else {             
      // nothing to do
    }     
}

void KSVDragList::dndMouseReleaseEvent( QMouseEvent* ) {
  pressed = FALSE;
}

void KSVDragList::mousePressEvent( QMouseEvent * _mouse ) {
  // Does the user want to select the icon ?
  if ( _mouse->button() == LeftButton ) {
    pressed = TRUE;
  } else if ( _mouse->button() == RightButton ) {
    pressed = FALSE;
    // set focus
    setFocus();
    emit sigRightPressed(this, getObject(_mouse->y()), QPoint(_mouse->x(),_mouse->y()));
  }

  focus = list.find(getObject(_mouse->y()));
  update();
}       

void KSVDragList::mouseDoubleClickEvent( QMouseEvent* _mouse ) {
  if (_mouse->button() == LeftButton) {
    emit sigDoubleClick( this, getObject(_mouse->y()), QPoint(_mouse->x(), _mouse->y()));
  }
}

void KSVDragList::paintEvent( QPaintEvent * ) {
  QPainter _p;
  int y = 0;
  QColor hiCol = kapp->winStyleHighlightColor();
  QColor blackCol = palette().active().text();
  QColor whiteCol = palette().active().base();

  _p.begin( &buffer );

  for (int i = 0; i < (int)list.count(); ++i) {
    KSVDragData* localO = list.at(i);
    QPixmap icon = localO->drawIcon();
    int paint_width = width() + 1;
    int paint_height = ENTRY_HEIGHT;
    QColor paint_back;
    QColor paint_text;
    int paint_y = y;

    if ( origin && (i == focus) ) {
      paint_height++;
      paint_back = hiCol;
      paint_text = whiteCol;
    } else {
      paint_back = whiteCol;
      paint_text = blackCol;
      if  ( origin && ((i == (focus + 1))) )
	paint_y++;
    }
    _p.setBackgroundMode( OpaqueMode );
    _p.fillRect( 0, paint_y, paint_width, paint_height, QBrush(paint_back));
    _p.setBackgroundMode( TransparentMode );

    // erase winFocusRect if any...
    //    _p.setPen(whiteCol);
    // _p.drawRect(1, y+1, 80-3, ENTRY_HEIGHT-1);
    
    // paint the label & icon
    _p.setPen( paint_text );
    _p.drawPixmap( QPoint(2,y+2), icon );
    _p.drawText( icon.width() + 4, y + 14, localO->getTitle() );
    
    // paint winFocusRect if current item has focus
    _p.setPen(blackCol);
    if (hasFocus() && (i == focus))
      _p.drawWinFocusRect(1, y + 1, 80 - 3, ENTRY_HEIGHT - 1 );
    else if (i == focus) {
      _p.setPen( hiCol );
      _p.drawLine( 1, y + ENTRY_HEIGHT, 80 - 3, y + ENTRY_HEIGHT );
    }

    // move next picture down...
    y += ENTRY_HEIGHT;
  }

  // erase the line at the bottom.. very "unclean"
  if ((focus != (count() - 1)) || !hasFocus())
    _p.fillRect( 0, y, width(), ENTRY_HEIGHT, QBrush(whiteCol));

  _p.end();
  
  bitBlt( this, 0,0, &buffer );               
}

void KSVDragList::slotDropEvent( KDNDDropZone* _zone) {
  QString droppedData = _zone->getData();
  if ( 
      //droppedData != QString(title) && 
      (_zone->getAcceptType() == _zone->getDataType()) ) {
    bool upper;
    
    QPoint akt_pos = mapFromGlobal(QCursor::pos());
    int position = list.find(getObject(akt_pos.y(), &upper));
    
    if (!upper)
      position++;

    KSVDragData* newO = new KSVDragData();
    newO->fromString(droppedData);

    // for removing elsehwere
    QString _old_rl = newO->getRunlevel();

    addItem(position, newO);

    emit sigDropOccured( _zone, this, upper );
    if (_old_rl != runlevel)
      emit sigDroppedFrom(_old_rl, newO);

  } else {
    // nothing happens
  }
}

void KSVDragList::setDefaultIcon( const char* _icon ) {
  defaultIcon = kil->loadIcon( QString( _icon ) );
  repaint(FALSE);
}

QPixmap KSVDragList::getDefaultIcon() {
  return defaultIcon;
}

void KSVDragList::resizeEvent( QResizeEvent* ) {
  buffer.resize(width(), height());
  buffer.fill();  
  emit sigResize( width(), height() );
}

KSVDragData* KSVDragList::getObject( int _y, bool* upperHalf) {
  if (( _y % ENTRY_HEIGHT ) < ENTRY_HEIGHT / 2)
    *upperHalf = TRUE;
  else
    *upperHalf = FALSE;

  return getObject(_y);
}

KSVDragData* KSVDragList::getObject(int _y) {
  if (!list.isEmpty()) {
    int _index = _y / ENTRY_HEIGHT;
    if (_index == count())
      _index--;
    return list.at( (uint)_index );
  } else
    return 0;
}

void KSVDragList::addItem(int _index, KSVDragData* _o) {
  _o->setIcon(&defaultIcon);
  _o->setRunlevel(runlevel);
  int pos = list.find(_o);
  if (pos == -1) {
    list.insert( (uint)_index, _o);
  } else {
    list.remove(list.at(pos));
    if (pos < _index)
      _index--;
    list.insert( (uint)_index, _o);
  }
  adjustSize();
  update();

  if (pos != _index) // if the item is not inserted into the same position as an existing one
    emit sigChanged(); 
}

int KSVDragList::count() {
  return (int)list.count();
}

void KSVDragList::keyPressEvent( QKeyEvent* e ) {
  int maxIndex = count() - 1;
  bool mod_key = (e->state() & ControlButton);

  switch( e->key() ) {                    // Look at the key code 

  case Key_Up:
    oldFocus = focus;
    if (mod_key) {
      if (focus > 0) {
	focus--;
	addItem(focus, list.at(oldFocus));
      }
    } else {
      if (focus > 0)
	focus--;
    }
    update();  
    emit sigScrollUp(focus);
    break; 
    
  case Key_Down:
    oldFocus = focus;
    if (mod_key) {
      if (focus < maxIndex) {
	focus++;
	addItem(focus+1, list.at(oldFocus));
      }
    } else {
      if (focus < maxIndex)
	focus++;;
    }
    update();
    emit sigScrollDown(focus);
    break;   

  case Key_F8:
    oldFocus = focus;
    if (removable) {
      if ( (!list.isEmpty()) && (focus <= maxIndex) ) {
	removeItem(list.at(focus));
	if (focus == maxIndex)
	  focus--;
	else if (focus > maxIndex)
	  focus = 0;
      }
      update();
    } else
      emit sigNotRemovable();
    break;
    
  case Key_F5:
    oldFocus = focus;
    emit sigCopy(this, list.at(focus));
    break;

  case Key_F6:
    oldFocus = focus;
    emit sigMove(this, list.at(focus));
    break;    

  default:                              // If not an interesting key,    
    e->ignore();                        // we don't accept the event 
    return;    
  }
}

void KSVDragList::removeItem( KSVDragData* _item ) {
  int i = list.find(_item);
  if (i != -1) {
    list.remove(list.at(i));
    update();
    if (list.find(_item) != i) // if it wasn't a drop on itself
      emit sigChanged();
  }
}

void KSVDragList::clear() {
  list.setAutoDelete(TRUE);
  int max = count();

  for(int i = 0; i < max; ++i) {
    
    // keep GUI alive
    qApp->processEvents();
  
    list.remove(list.at(0));
  }

  list.setAutoDelete(FALSE);
  update();

  // keep GUI alive
  qApp->processEvents();
}

void KSVDragList::setRunlevel( QString _run ) {
  runlevel = _run.copy();
}

void KSVDragList::update() {
  KDNDWidget::update();
  adjustSize();
  
  int max = count();
  int y = 0;
  
  // add tooltip
  for (int i = 0; i < max; i++) {
    QToolTip::add(this, QRect(0, y, 80, ENTRY_HEIGHT), list.at(i)->getFileName().data());
    y += ENTRY_HEIGHT;
  }
}

void KSVDragList::adjustSize() {
  setFixedSize(100, count() * ENTRY_HEIGHT + 1);
}

QString KSVDragList::getRunlevel() {
  return runlevel.copy();
}

void KSVDragList::setRemovable(bool _value) {
  removable = _value;
}

KSVDragData* KSVDragList::at(int _index) {
  if (_index < count())
    return list.at(_index);
  else
    return 0;
}

void KSVDragList::mouseMoveEvent( QMouseEvent* e ) {

  QPoint akt_pos = mapToGlobal(e->pos());
  
  emit mouseMove( akt_pos.x(), akt_pos.y() );

  KDNDWidget::mouseMoveEvent( e );
}

void KSVDragList::setOrigin( bool val ) {
  bool oldOrigin = origin;
  origin = val;
  if ( origin && (origin != oldOrigin) )
    emit sigOrigin( FALSE );
  else if ( oldOrigin )
    update();
  
  kapp->processEvents();
}

bool KSVDragList::isOrigin() {
  return origin;
}

void KSVDragList::focusInEvent( QFocusEvent* e ) {
  setOrigin(TRUE);

  KDNDWidget::focusInEvent( e );
}

void KSVDragList::focusOutEvent( QFocusEvent* e ) {
  // I thought I needed to do something here, but in the end didn't have to.
  // Maybe in the future, that's why I didn't remove this event.
  KDNDWidget::focusOutEvent( e );
}
