/*

    Requires the Qt widget libraries, available at no cost at
    http://www.troll.no

    Copyright (C) 1997 Peter Putzer
                       pputzer@edu.uni-klu.ac.at

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/     

#ifndef DRAGLIST_H
#define DRAGLIST_H

#include <qcolor.h>
#include <qpixmap.h>
#include <drag.h>
#include <kiconloader.h>
#include "ksv_dragdata.h"
#include "mylist.h"

class KSVDragList : public KDNDWidget {
  Q_OBJECT
    
 public:
  KSVDragList(QWidget* parent = 0, const char* name = 0);
  ~KSVDragList();

 protected:
  /* overridden */
  void dndMouseMoveEvent ( QMouseEvent* _mouse );
  void dndMouseReleaseEvent ( QMouseEvent* e );
  void mousePressEvent( QMouseEvent* e);
  void paintEvent ( QPaintEvent* e );
  void resizeEvent ( QResizeEvent* e );
  void keyPressEvent ( QKeyEvent* e );
  void mouseDoubleClickEvent ( QMouseEvent* e );
  void mouseMoveEvent( QMouseEvent* e );
  void update();
  void focusInEvent( QFocusEvent* e );
  void focusOutEvent( QFocusEvent* e );

  /* new */
  void adjustSize();
  KSVDragData* getObject(int _y, bool* _upper);
  KSVDragData* getObject(int _y);
  QPixmap getDefaultIcon();
  bool isOrigin();

  int focus;
  int oldFocus;
  bool pressed;
  bool removable;
  int press_x, press_y;
  KIconLoader* kil;
  KDNDDropZone* dz;
  QPixmap defaultIcon;
  QString runlevel;
  QPixmap buffer;
  KSVList list;	
  bool origin;

 public slots:
  void slotDropEvent( KDNDDropZone* _zone );
  void setDefaultIcon( const char* _icon );
  void setRunlevel( QString _run);
  void addItem( int _index, KSVDragData* _item );
  void removeItem( KSVDragData* _item );
  void setRemovable(bool _value);
  void clear();
  void setOrigin(bool val = TRUE);
  
 signals:
  void sigDropOccured( KDNDDropZone* _zone, KSVDragList* _w, bool _pos);
  void sigOrigin( bool val );

 signals:
  void sigDroppedFrom(QString _par, KSVDragData* _w);
  void sigChanged();
  void sigRightPressed(KSVDragList* _list, KSVDragData* _data, QPoint _pos);
  void sigResize(int _w, int _h);
  void sigScrollDown(int _focus);
  void sigScrollUp(int _focus);
  void sigNotRemovable();
  void sigDoubleClick(KSVDragList* _list, KSVDragData* _data, QPoint _pos);
  void sigCopy(KSVDragList* _list, KSVDragData* _data);
  void sigMove(KSVDragList* _list, KSVDragData* _data);
  void mouseMove( int _x, int _y );

 public:
  int count();
  QString getRunlevel();
  KSVDragData* at(int _index);

};

#endif
