/*

    Requires the Qt widget libraries, available at no cost at
    http://www.troll.no

    Copyright (C) 1997 Peter Putzer
                       pputzer@edu.uni-klu.ac.at

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/     

/****************************************************************
**
** kSysV
** Drag Widget
**
****************************************************************/

#include <qlabel.h>
#include <qpainter.h>
#include <qcursor.h>
#include <qbitmap.h>
#include <dragwidget.h>
#include <std.h>

// include meta-object file
#include "dragwidget.moc"

KSVDragWidget::KSVDragWidget(QWidget* parent = 0, const char* name = 0)
: KDNDWidget(parent, name){
  kil = new KIconLoader();
  dz = new KDNDDropZone(this, DndText);
  
  // connect the dropZone with my dropAction
  connect(dz, SIGNAL(dropAction(KDNDDropZone* )), this, SLOT(slotDropEvent( KDNDDropZone* )));
  
  // init (double-) buffer
  buffer = QPixmap();
}

KSVDragWidget::KSVDragWidget(QWidget* parent, const char* _title, const char* _icon, const char* _file)
: KDNDWidget(parent, _title) {
  kil = new KIconLoader();
  dz = new KDNDDropZone(this, DndText);
  
  iconName = _icon;
  fileName = _file;

  icon = kil->loadIcon( QString(iconName) );
  title = QString(_title);

  // connect the dropZone with my dropAction
  connect(dz, SIGNAL(dropAction(KDNDDropZone* )), this, SLOT(slotDropEvent( KDNDDropZone* )));

  // init (double-) buffer
  buffer = QPixmap(width(), height());

  // init class variables
  focus = FALSE;
  pressed = FALSE;
}

KSVDragWidget::~KSVDragWidget() {
    delete kil;
}

void KSVDragWidget::dndMouseMoveEvent ( QMouseEvent* _mouse ) {
  // 'pressed' is set in mousePressedEvent(...)
  if ( !pressed )
    return;
  
  int x = _mouse->pos().x();
  int y = _mouse->pos().y();
  
  if ( abs( x - press_x ) > Dnd_X_Precision || abs( y - press_y ) > Dnd_Y_Precision )
    {
      QString data = title.copy();
      data += "|";
      data += iconName;
      data += "|";
      data += fileName;
      data += "|";
      if (parentWidget() != 0)
	data += parentWidget()->name();
      
      
      QPoint p = mapToGlobal( _mouse->pos() );
      QPixmap pixmap = icon;
      //    QPixmap pixmap = buffer;
      //       pixmap.setMask( pixmap.createHeuristicMask() );
      int dx = - pixmap.width() / 2;
      int dy = - pixmap.height() / 2;
      
      startDrag( new KDNDIcon( pixmap, p.x() + dx, p.y() + dy ), data.data(), data.length(), DndText, dx, dy );
      pressed = FALSE;
    }
  else
    {             
      
    }     
}

void KSVDragWidget::dndMouseReleaseEvent( QMouseEvent* ) {
  pressed = FALSE;
}

void KSVDragWidget::mousePressEvent( QMouseEvent * _mouse ) {
  {
    // Does the user want to select the icon ?
    if ( _mouse->button() == LeftButton )
      {
	pressed = TRUE;
      } else if ( _mouse->button() == RightButton ) {
	pressed = FALSE;
	emit sigRightPressed(this, QPoint(_mouse->x(),_mouse->y()));
	
      }
  }       
  
}

void KSVDragWidget::paintEvent( QPaintEvent * )
{
  QPainter _p;
  
  _p.begin( &buffer );
  _p.setBackgroundMode( TransparentMode );
  _p.drawPixmap( QPoint(0,0), icon );
  _p.setPen( QColor(0,0,0) );
  _p.drawText( icon.width() + 1, 15, title );
  _p.end();

  bitBlt( this, 0,0, &buffer );               
  

//   _p.begin( this );
//   _p.drawPixmap( QPoint(0,0), buffer );
//   _p.end();

}

void KSVDragWidget::slotDropEvent( KDNDDropZone* _zone) {
  QString droppedData = _zone->getData();
  if ( droppedData != QString(title) && (_zone->getAcceptType() == _zone->getDataType()) ) {
    
    
    //     *MyColor = blue;
    //     slotSetTitle(QString(droppedData.data()));
    
    //     repaint(FALSE);
    bool where;
    
    
    QPoint akt_pos = mapFromGlobal(QCursor::pos());
    if (akt_pos.y() > (height() / 2))
      where = FALSE;
    else
      where = TRUE;
    emit sigDropOccured( _zone, this, where );
  } else {
    // nothing happens
  }
}

void KSVDragWidget::slotSetIcon( const char* _icon ) {
  icon = kil->loadIcon( QString( _icon ) );
  iconName = _icon;
  repaint(FALSE);
}

void KSVDragWidget::slotSetTitle( const char* _title ) {
  title = _title;
  repaint(FALSE);
}

void KSVDragWidget::slotSetFileName( const char* _file ) {
  fileName = _file;
}

QString KSVDragWidget::getTitle() {
  return title;
}

QString KSVDragWidget::getFileName() {
  return fileName;
}

QString KSVDragWidget::getIconName() {
  return iconName;
}

void KSVDragWidget::resizeEvent( QResizeEvent* ) {
  buffer.resize(width(), height());
  buffer.fill();  
}
