/*

    Requires the Qt widget libraries, available at no cost at
    http://www.troll.no

    Copyright (C) 1997 Peter Putzer
                       pputzer@edu.uni-klu.ac.at

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/     

#ifndef DRAGWIDGET_H
#define DRAGWIDGET_H

#include "qcolor.h"
#include "qpixmap.h"
#include "drag.h"
#include "kiconloader.h"

class KSVDragWidget : public KDNDWidget
{
  Q_OBJECT
    
 public:
  KSVDragWidget(QWidget* parent = 0, const char* name = 0);
  KSVDragWidget(QWidget* parent, const char* _title, const char* _icon, const char* _file);
  
  // destructor
  ~KSVDragWidget();

 protected: 
  friend class KSVList;
  friend class KSVWidgetList;
  void dndMouseMoveEvent ( QMouseEvent * _mouse );
  void dndMouseReleaseEvent ( QMouseEvent * );
  void mousePressEvent( QMouseEvent *);
  void paintEvent ( QPaintEvent* e );
  void resizeEvent ( QResizeEvent* e );
    
  bool focus;
  bool pressed;
  int press_x, press_y;
  KIconLoader* kil;
  KDNDDropZone* dz;

  QString title;
  QPixmap icon;
  QString iconName;
  QString fileName;

 public slots:
  void slotDropEvent( KDNDDropZone* _zone);
  void slotSetIcon( const char* _icon );
  void slotSetTitle( const char* _title );
  void slotSetFileName( const char* _file );
  
 signals:
  void sigDropOccured( KDNDDropZone* _zone, KSVDragWidget* _w, bool _pos);
  void sigRightPressed( KSVDragWidget* _w, QPoint _mpos );
  
 public:
  QString getTitle();
  QString getFileName();
  QString getIconName();

 private:
  QPixmap buffer;
};

#endif
