/*

    Requires the Qt widget libraries, available at no cost at
    http://www.troll.no

    Copyright (C) 1997 Peter Putzer
                       pputzer@edu.uni-klu.ac.at

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/     

#include <qpainter.h>
#include <qpixmap.h>
#include <qbitmap.h>
#include "ksv_dragdata.h"
#include <kapp.h>

// include meta-object code
#include "ksv_dragdata.moc"

// define macros
#define ENTRY_HEIGHT 19
#define ENTRY_WIDTH 80

KSVDragData::KSVDragData() : QObject() {
  title = "Empty";
  icon = QPixmap();
  focus = QPixmap(ENTRY_WIDTH, ENTRY_HEIGHT+1);
  noFocus = QPixmap(ENTRY_WIDTH, ENTRY_HEIGHT+1);
}

KSVDragData::~KSVDragData() {
  // nothing yet
}

QString KSVDragData::toString() {
  QString data = title.copy();
  data += "|";
  data += "foo icon"; //icon;
  data += "|";
  data += path;
  data += "|";
//   if (parentWidget() != 0)
//     data += parentWidget()->name();
  data += runlevel;
  return data;
}

void KSVDragData::fromString(QString _data) {
  int _firstP = _data.find("|");
  int _secondP = _data.find("|", _firstP + 1);
  int _thirdP = _data.find("|", _secondP + 1);
  
  QString d_Title = _data.mid(0, _firstP );
  QString d_Icon = _data.mid(_firstP + 1, _secondP - _firstP - 1);
  QString d_Path = _data.mid(_secondP + 1, _thirdP - _secondP -1);
  QString d_Parent = _data.mid(_thirdP + 1, _data.length() - _thirdP - 1);

  // assign values to members
  title = d_Title;
  // icon = d_Icon;  //... not here
  path = d_Path;
  runlevel = d_Parent;

  updateIcons();
}

QPixmap KSVDragData::drawIcon() {
  return  icon;
}

void KSVDragData::setIcon(QPixmap* _pix) {
  icon = *_pix;
  updateIcons();
}

void KSVDragData::setTitle(QString _str) {
  title = _str.copy();
  updateIcons();
}

QString KSVDragData::getTitle() {
  return title.copy();
}

QString KSVDragData::getRunlevel() {
  return runlevel.copy();
}

QString KSVDragData::getFileName() {
  return path.copy();
}

void KSVDragData::setRunlevel(QString _str) {
  runlevel = _str.copy();
}

void KSVDragData::updateIcons() {
  QPainter p;
  QColor hiCol = kapp->winStyleHighlightColor();
  QColor blackCol = kapp->palette()->active().text();
  QColor whiteCol = kapp->palette()->active().base();
  QFont textFont = *kapp->font();
  textFont.setPointSize(10);

  p.begin( &focus );
  // set defaults
  p.setFont( textFont );
  
  int paint_width = ENTRY_WIDTH + 1;
  int paint_height = ENTRY_HEIGHT;
  QColor paint_back;
  QColor paint_text;
  
  paint_height++;
  paint_back = hiCol;
  paint_text = whiteCol;
  
  p.setBackgroundMode( OpaqueMode );
  p.fillRect( 0, 0, paint_width, paint_height, QBrush(paint_back));
  p.setBackgroundMode( TransparentMode );

  // paint the label & icon
  p.setPen( paint_text );
  p.drawPixmap( QPoint(2,2), icon );
  p.drawText( icon.width() + 4, 14, getTitle() );
 
  // paint winFocusRect
  p.setPen(blackCol);
  p.drawWinFocusRect(1, 1, ENTRY_WIDTH - 3, ENTRY_HEIGHT - 1 );

  p.end();

  p.begin( &noFocus );
  
  // set defaults
  p.setFont( textFont );
  
  paint_width = ENTRY_WIDTH + 1;
  paint_height = ENTRY_HEIGHT;
 
  paint_back = whiteCol;
  paint_text = blackCol;
  //   if (hasFocus() && ((i == (focus + 1))))
  //     paint_y++;
  p.setBackgroundMode( OpaqueMode );
  p.fillRect( 0, 0, paint_width, paint_height, QBrush(paint_back));
  p.setBackgroundMode( TransparentMode );

  p.drawPixmap( QPoint(2,2), icon);
  p.drawText( icon.width() + 4, 14, getTitle() );
  
  p.end();
}

QPixmap* KSVDragData::drawFocusIcon() {
  return &focus;
}

QPixmap* KSVDragData::drawNoFocusIcon() {
  return &noFocus;
}
