/*

    Requires the Qt widget libraries, available at no cost at
    http://www.troll.no

    Copyright (C) 1997 Peter Putzer
                       pputzer@edu.uni-klu.ac.at

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/     

/****************************************************************
**
** kSysV
** TopLevelWidget
**
****************************************************************/

#include <ksvtop.h>
#include <qpopmenu.h>
#include <qmsgbox.h>
#include <qkeycode.h>
#include <kapp.h>
#include <kmsgbox.h>
#include "version.h"

// include meta-object file
#include "ksvtop.moc"

KSVTopLevel::KSVTopLevel() : KTopLevelWidget() {
  _kil = new KIconLoader();
  _conf = kapp->getConfig();
  
  _BorderWidth = 1;
  setFrameBorderWidth(_BorderWidth);
  
  _menu = new KMenuBar(this);
  initMenu();
  _menu->show();
  setMenu(_menu);          
  
  _tools = new KToolBar(this);
  initToolBar();
  _tools->show();
  addToolBar(_tools);
  
  _status = new KStatusBar(this);
  initStatusBar();
  _status->show();
  setStatusBar(_status);
  
  _view = new KSysV(this);
  _view->show();
  setView(_view, TRUE);

  // ensure that the statusbar gets updated correctly
  connect(_view, SIGNAL(sigRun(QString)), this, SLOT(slotUpdateRunning(QString)));
  connect(_view, SIGNAL(sigStop()), this, SLOT(slotClearRunning()));
  connect(_view, SIGNAL(sigNotRemovable()), this, SLOT(slotScriptsNotRemovable()));
 
  // ensure that the window gets resized correctly if toolbar
  // or menubar floats or the view changes size
  connect(_menu, SIGNAL(moved(menuPosition)), this, SLOT(slotMoved()));
  connect(_tools, SIGNAL(moved(BarPosition)), this, SLOT(slotMoved()));
  connect(_view, SIGNAL(sigUpdateParent()), this, SLOT(slotMoved()));
 
  menuNormHeight = _menu->height();
  toolsNormHeight = _tools->height();
  
  // update geometry
  slotMoved();
  
  // init class members
  _prefs = new KSVPrefDlg(this, "Preferences");
  _prefs->hide();
  
  _saveOptions = FALSE;
  _changed = FALSE;
  connect(_view, SIGNAL(sigChanged()), this, SLOT(slotChanged()));
  
  // init _view according to saved preferences
  slotReadConfig();
  _view->reInit();

  // restore "unchanged" state
  _changed = FALSE;
  
  // for session management.. well sort of
  setUnsavedData(FALSE);
  
  // disable "Abort Changes"
  _tools->setItemEnabled(1, FALSE);
  _FileMenu->setItemEnabled(0, FALSE);
  // disable "Save Changes"
  _tools->setItemEnabled(2, FALSE);
  _FileMenu->setItemEnabled(1, FALSE);
  
  // update statusbar
  _status->changeItem("", 2);
  
}

KSVTopLevel::~KSVTopLevel() {
  delete _kil;
}

void KSVTopLevel::initMenu() {
  
  // generate menu
  _FileMenu = new QPopupMenu();
  _FileMenu->insertItem(i18n("&Clear Changes"), this, SLOT(slotClearChanges()));
  _FileMenu->insertItem(i18n("&Accept Changes"), this, SLOT(slotAcceptChanges()));
  _FileMenu->setId(0,0);
  _FileMenu->setId(1,1);
  _FileMenu->setItemEnabled(0, FALSE);
  _FileMenu->setItemEnabled(1, FALSE);
  _FileMenu->insertSeparator();
  _FileMenu->insertItem(i18n("E&xit"), this, SLOT(slotCloseMain()), CTRL+Key_Q);

  _EditMenu = new QPopupMenu();
  _EditMenu->insertItem(i18n("C&ut"), this, SLOT(editCut()), CTRL+Key_X);
  _EditMenu->insertItem(i18n("&Copy"), this, SLOT(editCopy()), CTRL+Key_C);
  _EditMenu->insertItem(i18n("&Paste"), this, SLOT(editPaste()), CTRL+Key_V);
  _EditMenu->setId(0,0);
  _EditMenu->setId(1,1);
  _EditMenu->setId(2,2);
  _EditMenu->setItemEnabled(0, FALSE);
  _EditMenu->setItemEnabled(1, FALSE);
  _EditMenu->setItemEnabled(2, FALSE);

  _OptionsMenu = new QPopupMenu();
  _OptionsMenu->setCheckable(TRUE);
  _OptionsMenu->insertItem(i18n("&Configure Paths..."), this, SLOT(slotShowConfig()));
  _OptionsMenu->insertItem(i18n("&Show Script Output"), this, SLOT(slotToggleOut()));
  _OptionsMenu->insertSeparator();
  _OptionsMenu->insertItem(i18n("S&ave Options on Exit"), this, SLOT(slotSaveOptions()));
  _OptionsMenu->setId(0,0);
  _OptionsMenu->setId(1,1);
  _OptionsMenu->setId(3,2);
  _OptionsMenu->setItemChecked(1, TRUE);
  _OptionsMenu->setItemChecked(2, FALSE);

  _HelpMenu = new QPopupMenu();
  _HelpMenu->insertItem(_kil->loadIcon("ksysv_help.xpm"), i18n("&Contents"), this, SLOT(slotInvokeHelp()), Key_F1);
  _HelpMenu->insertSeparator();
  _HelpMenu->insertItem(_kil->loadIcon("ksysv_info.xpm"), i18n("&About..."), this, SLOT(slotShowAbout()));
  _HelpMenu->insertItem(i18n("About &Qt..."), this, SLOT(slotAboutQt()));
    
  _menu->insertItem(i18n("&File"), _FileMenu);
  _menu->insertItem(i18n("&Edit"), _EditMenu);
  _menu->insertItem(i18n("&Options"), _OptionsMenu);
  _menu->insertSeparator();
  _menu->insertItem(i18n("&Help"), _HelpMenu);
}

void KSVTopLevel::initToolBar() {
  QPixmap _icon = _kil->loadIcon(QString("exit.xpm"));
  _tools->insertButton(_icon, 0, SIGNAL(clicked(int)), this, SLOT(slotCloseMain()), TRUE, i18n("Exit ") + KSysV::getAppName());
  _tools->insertSeparator();
  
  _icon = _kil->loadIcon(QString("clear.xpm"));
  _tools->insertButton(_icon, 1, SIGNAL(clicked(int)), this, SLOT(slotClearChanges()), FALSE, i18n("Abort changes"));
  
  _icon = _kil->loadIcon(QString("save.xpm"));
  _tools->insertButton(_icon, 2, SIGNAL(clicked(int)), this, SLOT(slotAcceptChanges()), FALSE, i18n("Save changes"));
  _tools->insertSeparator();
   
  _icon = _kil->loadIcon(QString("toggle.xpm"));
  _tools->insertButton(_icon, 3, SIGNAL(clicked(int)), this, SLOT(slotToggleOut()), TRUE, i18n("Toggle script output"));
  _tools->setToggle(3, TRUE);
  _tools->setButton(3, TRUE);
  _tools->insertSeparator();
  
  _icon = _kil->loadIcon(QString("help.xpm"));
  _tools->insertButton(_icon, 4, SIGNAL(clicked(int)), this, SLOT(slotInvokeHelp()), TRUE, i18n("Help"));
  
  _icon = _kil->loadIcon(QString("info.xpm"));
  _tools->insertButton(_icon, 5, SIGNAL(clicked(int)), this, SLOT(slotShowAbout()), TRUE, i18n("About ") + KSysV::getAppName());
}

void KSVTopLevel::resizeWidgets() {
  int heightToResizeTo = 0;
  int widthToResizeTo = 0;
  
  if ( (_tools->barPos() == KToolBar::Top) | (_tools->barPos() == KToolBar::Bottom) )
    heightToResizeTo += toolsNormHeight;
  if (_menu->menuBarPos() != KMenuBar::Floating)
    heightToResizeTo += menuNormHeight;
  
  if ( (_tools->barPos() == KToolBar::Left) | (_tools->barPos() == KToolBar::Right) )
    widthToResizeTo += _tools->width();
  
  //	if ( (_status->))
  heightToResizeTo += _status->height();
  
  heightToResizeTo += _view->height();
  heightToResizeTo += 2 * _BorderWidth;
  
  widthToResizeTo += _view->width();
  widthToResizeTo += 2 * _BorderWidth;
  
  setFixedSize(widthToResizeTo, heightToResizeTo);	
}

void KSVTopLevel::slotMoved() {
  resizeWidgets();
  updateRects();
}

void KSVTopLevel::closeEvent( QCloseEvent *e )    {        
  int _res;
  if (_changed) {
    _res = QMessageBox::warning(kapp->topWidget(), i18n("Quit"), i18n("There are unsaved changes.\nAre you sure you want to quit?"), 
				i18n("Yes, quit!"), i18n("No, don't quit."), 0, 1, 1);
  } else {
    _res = 0;
  }
    
  if (_res == 0) {
    if (_saveOptions)
      slotWriteConfig();
    delete _FileMenu;
    delete _OptionsMenu;
    delete _HelpMenu;
    delete _menu;
    delete _tools;
    e->accept();
  } else {
    e->ignore();
  }
}

void KSVTopLevel::slotCloseMain() {
  close();
}

void KSVTopLevel::slotInvokeHelp() {
  kapp->invokeHTMLHelp(QString("ksysv/ksysv.html"),QString());
}

void KSVTopLevel::slotShowAbout() {
  QString ksysv = KSysV::getAppName();
  
  QMessageBox* aboutKSV = new QMessageBox( kapp->topWidget(), "aboutBox");

  aboutKSV->setText( (((ksysv + i18n("\nVersion ")) + KSYSV_VERSION) + "\n\n(c) 1997, 1998") + " Peter Putzer\npputzer@edu.uni-klu.ac.at" );
  aboutKSV->setCaption( i18n("About ") + ksysv );
  aboutKSV->setIconPixmap( _kil->loadIcon("ksysv_logo.xpm") );
  aboutKSV->setButtonText( 0, i18n("OK") );
    
  aboutKSV->exec();
  
  // reclaim memory
  delete aboutKSV;
  aboutKSV = 0;
  
//   QMessageBox::about(kapp->topWidget(), i18n("About ") + ksysv, (((ksysv +  
// 								 i18n("\nVersion ")) + KSYSV_VERSION) + "\n\n(c) 1997") + 
// 		     " Peter Putzer\npputzer@edu.uni-klu.ac.at");
}

void KSVTopLevel::slotClearChanges() {
  if (_changed) {
    int _res = QMessageBox::warning(kapp->topWidget(), i18n("Clear Changes"), i18n("Do you really want to clear all unsaved changes?"),
				    i18n("Yes"), i18n("No"), 0, 1, 1);
    if (_res == 0) {

      // keep GUI alive
      qApp->processEvents();
    
      _view->reInit();
      _changed = FALSE;

      // for session management.. well sort of
      setUnsavedData(FALSE);

      // disable "Abort Changes"
      _tools->setItemEnabled(1, FALSE);
      _FileMenu->setItemEnabled(0, FALSE);
      // disable "Save Changes"
      _tools->setItemEnabled(2, FALSE);
      _FileMenu->setItemEnabled(1, FALSE);
      
        // update statusbar
      _status->changeItem("", 2);
  
    }
  }
}

void KSVTopLevel::slotAcceptChanges() {
    int _res = QMessageBox::warning(kapp->topWidget(), i18n("Save Changes"), 
				    i18n("You're about to save the changes made to your init " 
				       "configuration.\nPlease realize that wrong settings could "  
				       "result in your system being unbootable.\n\n"
				       "Do you wish to continue?"), i18n("Yes, commit changes!"), i18n("No, don't save changes."), 0, 1, 1);
    if (_res == 0) {
      _view->slotWriteSysV();
      _changed = FALSE;
      
      // for session management.. well sort of
      setUnsavedData(FALSE);

      // disable "Abort Changes"
      _tools->setItemEnabled(1, FALSE);
      _FileMenu->setItemEnabled(0, FALSE);
      // disabel "Save Changes"
      _tools->setItemEnabled(2, FALSE);
      _FileMenu->setItemEnabled(1, FALSE);
      
      // update statusbar
      _status->changeItem("", 2);
    }
}

void KSVTopLevel::initStatusBar() {
  _status->insertItem("                                                            "	\
		      "                                                            "	\
		      "                                                            "	\
		      "         ", 1);
  _status->insertItem("            ", 2);
}

void KSVTopLevel::slotShowConfig() {
  int res = _prefs->exec();
  if (res == QDialog::Accepted) {
    slotWriteConfig();
    _view->reInit();
    
    // set to "unchanged" status
    _changed = FALSE;
    // update statusbar
    _status->changeItem("", 1);
    
    // for session management.. well sort of
    setUnsavedData(FALSE);
    
    // disable "Abort Changes"
    _tools->setItemEnabled(1, FALSE);
    _FileMenu->setItemEnabled(0, FALSE);
    // disable "Save Changes"
    _tools->setItemEnabled(2, FALSE);
    _FileMenu->setItemEnabled(1, FALSE);
  } else
    slotCancelConfig();
}

void KSVTopLevel::slotChanged() {
  _changed = TRUE;
  // for session management
  setUnsavedData(TRUE);
  
  // set discard changes enabled
  _tools->setItemEnabled( 1, TRUE);
  _FileMenu->setItemEnabled(0, TRUE);
  // set save changes enabled
  _tools->setItemEnabled( 2, TRUE);
  _FileMenu->setItemEnabled(1, TRUE);

  // update statusbar
  _status->changeItem(i18n("Changed"), 2);
  // clear field 1
  _status->changeItem("", 1);
}

void KSVTopLevel::slotReadConfig() {
  QString _scriptPath;
  QString _rlPath;
  
  _conf->setGroup( QString("Path Settings") );
  _scriptPath = _conf->readEntry(QString("ScriptPath"), "/etc/rc.d/init.d");
  _rlPath = _conf->readEntry(QString("RunlevelPath"), "/etc/rc.d");
  
  _conf->setGroup( QString("Other Settings") );
  if (!_conf->readNumEntry(QString("ScriptOutput"), TRUE))
    slotToggleOut();
  if (_conf->readNumEntry(QString("SaveOptions"), FALSE))
    slotSaveOptions();
  _prefs->_scriptPath->setText(_scriptPath.data());
  _prefs->_rlPath->setText(_rlPath.data());
	  
  _view->setPathToScripts(_scriptPath);
  _view->setPathToRLs(_rlPath);
}

void KSVTopLevel::slotWriteConfig() {
  QString _scriptPath;
  QString _rlPath;
  
  _scriptPath = QString(_prefs->_scriptPath->text());
  _rlPath = QString(_prefs->_rlPath->text());
  
  _view->setPathToScripts(_scriptPath);
  _view->setPathToRLs(_rlPath);
  
  _conf->setGroup( QString("Path Settings") );	
  _conf->writeEntry(QString("ScriptPath"), _scriptPath);
  _conf->writeEntry(QString("RunlevelPath"), _rlPath);
  
  _conf->setGroup( QString("Other Settings") );
  _conf->writeEntry(QString("ScriptOutput"), _OptionsMenu->isItemChecked(1));

  // save screen geometry (well, position that is)
  _conf->setGroup( QString("Geometry"));
  _conf->writeEntry(QString("X-Position"), x());
  _conf->writeEntry(QString("Y-Position"), y());

  _conf->sync();
  
  // _view->reInit();
  _changed = FALSE;
  // update statusbar
  _status->changeItem("", 1);

  // for session management.. well sort of
  setUnsavedData(FALSE);

  // disable "Abort Changes"
  _tools->setItemEnabled(1, FALSE);
  _FileMenu->setItemEnabled(0, FALSE);
  // disable "Save Changes"
  _tools->setItemEnabled(2, FALSE);
  _FileMenu->setItemEnabled(1, FALSE);
}

void KSVTopLevel::slotCancelConfig() {
  QString _scriptPath;
  QString _rlPath;

  _prefs->_scriptPath->setText(_view->pathToScripts.data());
  _prefs->_rlPath->setText(_view->pathToRLs.data());
}

void KSVTopLevel::slotAboutQt() {
  QMessageBox::aboutQt(kapp->topWidget(), i18n("About Qt"));
}

void KSVTopLevel::slotToggleOut() {
#define OM _OptionsMenu
  if (OM) {
    OM->setItemChecked(1, !OM->isItemChecked(1));
    _tools->setButton(3, OM->isItemChecked(1));
    _view->slotToggleOut();
  }
#undef OM
}

void KSVTopLevel::slotSaveOptions() {
#define OM _OptionsMenu
  if (OM) {
    OM->setItemChecked(2, !OM->isItemChecked(2));
    _saveOptions = !_saveOptions;
    _conf->setGroup( QString("Other Settings") );
    _conf->writeEntry(QString("SaveOptions"), _OptionsMenu->isItemChecked(2));
    _conf->sync();
  }
#undef OM
}

void KSVTopLevel::slotUpdateRunning(QString _text) {
  _status->changeItem(_text.data(), 1);
}

void KSVTopLevel::slotClearRunning() {
  _status->changeItem("", 1);
}

void KSVTopLevel::slotScriptsNotRemovable() {
  _status->changeItem(i18n("Cannot remove script from list"), 1);
}

void KSVTopLevel::editCut() { 
  // nix
}

void KSVTopLevel::editCopy() {
  // noch weniger
}

void KSVTopLevel::editPaste() {
  // absolut ueberhaupt nix
}
