/*

    Requires the Qt widget libraries, available at no cost at
    http://www.troll.no

    Copyright (C) 1997 Peter Putzer
                       pputzer@edu.uni-klu.ac.at

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/     

/****************************************************************
**
** kSysV
** Main Widget
**
****************************************************************/

#include <dir.h>
#include <unistd.h>
#include <qlabel.h>
#include <qscrbar.h>
#include <qtooltip.h>
#include <qpopmenu.h>
#include <qkeycode.h>
#include <qmsgbox.h>
#include <qregexp.h>
#include <kiconloader.h>
#include <kprocess.h>
#include <kapp.h>
#include "ksysv.h"
#include "ksv_dragdata.h"
#include "new_dwlst.h"
#include "copydialog.h"

// include the meta-object file
#include "ksysv.moc"

KSysV::KSysV(QWidget* parent = 0, const char* name = 0) : QWidget(parent, name) {
  kil = new KIconLoader();

  KDNDDropZone* dz = new KDNDDropZone(this, DndText);
  connect( dz, SIGNAL(dropAction(KDNDDropZone*)), this, SLOT(slotDropAction(KDNDDropZone*)));
  
  trash = new KSVTrash(this, "Trash");

  initLList();
  
  // some defaults... well they'll be overridden almost immediately, but... :)
  pathToScripts = QString("/etc/rc.d/init.d");
  pathToRLs = QString("/etc/rc.d");
  
  // set trash geometry after scripts exists
  trash->setGeometry(5, scripts->height() + 44, scripts->width(), 45);
  trash->show();

  // other class members
  _menu = 0;
  _currentW = 0;
  showOut = TRUE;
  
  setFixedSize(4 + 105 * 8, realHeight);
}

KSysV::~KSysV() {
  // I do not know what's going on here.
  // but it seems like I can't delete
  // the popup menu (if it's still there)
  // in here... *wonders*

//   if (_menu) {
//     delete _menu;
//     _menu = 0;
//   }
  delete kil;
}

void KSysV::slotDropAction( KDNDDropZone* ) {
  // nothing  
}

void KSysV::initLList() {
  int _x = 5;
  int _y = 40;
  int _h_dist = 105;
  int _v_dist = 105;
    
  QLabel* _sl = new QLabel( i18n("Available"), this );
  QLabel* _sl2 = new QLabel( i18n("&Services"), this );
  _sl->move(_x, 5);
  _sl->resize(_sl->sizeHint());
  //  _sl->setGeometry(_x, 5, _h_dist, 30);
  _sl->show();
  
  QFont sl2_font = QFont(kapp->generalFont);
  sl2_font.setBold(TRUE);
  _sl2->move(_x, _y - 35 + 15);
  _sl2->resize(_sl2->sizeHint());
  _sl2->setFont(sl2_font);
  _sl2->show();


  scripts = new KSVWidgetList(this, "Scripts");
  scripts->setGeometry(_x, _y, 110, _v_dist);
  scripts->setWidth(80);
  scripts->setMaxVisible(17);
  scripts->setRemovable(FALSE);
  scripts->setDropTarget(FALSE);
  scripts->setDefaultIcon("ksysv_exec.xpm");

  // setBuddy
  _sl2->setBuddy(scripts);

  // menus
  connect( scripts, SIGNAL(sigPopupMenu(KSVDragData*, QPoint, bool)), this, SLOT(slotPopupScriptsMenu(KSVDragData*, QPoint, bool)) );
  connect( scripts, SIGNAL(sigNotRemovable()), this, SLOT(slotScriptsNotRemovable()));
  // doubleclick
  connect( scripts, SIGNAL(sigDoubleClick(KSVDragData*, QPoint)), this, SLOT(slotDoubleClick(KSVDragData*, QPoint)));
  connect( scripts, SIGNAL(sigCopy(KSVDragData*)), this, SLOT(slotCopy(KSVDragData*)));

  _x += _h_dist;

  for(int i = 0; i<7; ++i) {
    
    // create QString for label
    QString _label;
    _label.sprintf(i18n("Runlevel &%i"), i);
    // and for the name
    QString _name;
    _name.sprintf(i18n("Runlevel %i"), i);

    QLabel* _heading = new QLabel(_label, this);
    QLabel* _start = new QLabel(i18n("Start"), this);
    
    QFont heading_font = QFont(kapp->generalFont);
    heading_font.setBold(TRUE);
    
    //     _heading->setGeometry(_x, _y - 35, _h_dist, 10);
    _heading->move(_x, _y - 35);
    _heading->resize(_heading->sizeHint());
    _heading->setFont(heading_font);
    _heading->show();
    _start->setGeometry(_x, _y + 10 + 5 - 35, _h_dist, 15);
    _start->show();
    
    // create the "START" list:

    startRL[i] = new KSVWidgetList(this, (_name + " START").data());
    startRL[i]->setGeometry(_x, _y, 110, _v_dist);
    startRL[i]->setMaxVisible(9);
    startRL[i]->setWidth(80);
    startRL[i]->setDefaultIcon("ksysv_start.xpm");
  
    QLabel* _stop = new QLabel(i18n("Stop"), this);
    _stop->setGeometry(_x, _y + startRL[i]->height() + 5, _h_dist, 15);
    _stop->show();

    // create the "STOP" list:

    stopRL[i] = new KSVWidgetList(this, (_name + " STOP").data());
    stopRL[i]->setGeometry(_x, _y + startRL[i]->height() + 5 + _stop->height() + 5, 110, _v_dist);
    stopRL[i]->setMaxVisible(9);
    stopRL[i]->setWidth(80);
    stopRL[i]->setDefaultIcon("ksysv_stop.xpm");

    // set the buddy widget for the "Runlevel %i" label... => the corresponding runlevel
    _heading->setBuddy(startRL[i]);

    // when a DragWidget is moved to another WidgetList, it gets removed & deleted correctly
    // if you connect the following signals and slots:
    
    connect( startRL[i], SIGNAL(sigDroppedFrom(QString, KSVDragData*)), scripts, SLOT(slotRemoveAfterDrop(QString, KSVDragData*)));
    connect( stopRL[i], SIGNAL(sigDroppedFrom(QString, KSVDragData*)), scripts, SLOT(slotRemoveAfterDrop(QString, KSVDragData*)));
    connect( trash, SIGNAL(sigDroppedFrom(QString, KSVDragData*)), startRL[i], SLOT(slotRemoveAfterDrop(QString, KSVDragData*)));
    connect( trash, SIGNAL(sigDroppedFrom(QString, KSVDragData*)), stopRL[i], SLOT(slotRemoveAfterDrop(QString, KSVDragData*)));
    
    _x += _h_dist;
    
    startRL[i]->show();
    stopRL[i]->show();

    // set height
    realHeight = 5 + _heading->height() + 5 + _start->height() + 5 + startRL[i]->height()
      + 5 + _stop->height() + 5 + stopRL[i]->height() + 5;
  }


  // add text-diplay widget
  textDisplay = new QMultiLineEdit(this, "TextDisplayWidget");
  textDisplay->setReadOnly(TRUE);
  textDisplay->setAutoUpdate(TRUE);
  textDisplay->move(5, realHeight);
  textDisplay->resize(8*105 - 3, 100);
  textDisplay->setFocusPolicy(NoFocus);
  textDisplay->show();
  
  realHeight += 5 + textDisplay->height();
 
  for (int i = 0; i < 7; ++i) {
    for (int j = 0; j < 7; ++j) {
      connect( startRL[i], SIGNAL(sigDroppedFrom(QString, KSVDragData*)), startRL[j], SLOT(slotRemoveAfterDrop(QString, KSVDragData*)) );
      connect( startRL[i], SIGNAL(sigDroppedFrom(QString, KSVDragData*)), stopRL[j], SLOT(slotRemoveAfterDrop(QString, KSVDragData*)) );
      connect( stopRL[i], SIGNAL(sigDroppedFrom(QString, KSVDragData*)), startRL[j], SLOT(slotRemoveAfterDrop(QString, KSVDragData*)) );
      connect( stopRL[i], SIGNAL(sigDroppedFrom(QString, KSVDragData*)), stopRL[j], SLOT(slotRemoveAfterDrop(QString, KSVDragData*)) );
      
      // capturing mouse moves
      connect( startRL[i], SIGNAL(mouseMove(int,int)), startRL[j], SLOT(autoScroll(int,int)) );
      connect( startRL[i], SIGNAL(mouseMove(int,int)), stopRL[j], SLOT(autoScroll(int,int)) );
      connect( stopRL[i], SIGNAL(mouseMove(int,int)), startRL[j], SLOT(autoScroll(int,int)) );
      connect( stopRL[i], SIGNAL(mouseMove(int,int)), stopRL[j], SLOT(autoScroll(int,int)) );

      // connect origin sigs & slots
      if (i != j) 
	connect( startRL[i], SIGNAL(sigOrigin(bool)), startRL[j], SLOT(setOrigin(bool)) );
      connect( startRL[i], SIGNAL(sigOrigin(bool)), stopRL[j], SLOT(setOrigin(bool)) );
      connect( stopRL[i], SIGNAL(sigOrigin(bool)), startRL[j], SLOT(setOrigin(bool)) );
      if (i != j)
	connect( stopRL[i], SIGNAL(sigOrigin(bool)), stopRL[j], SLOT(setOrigin(bool)) );
      
    }
    
    // capturing mouse moves ... also for "Scripts"
    connect( scripts, SIGNAL(mouseMove(int,int)), startRL[i], SLOT(autoScroll(int,int)) );
    connect( scripts, SIGNAL(mouseMove(int,int)), stopRL[i], SLOT(autoScroll(int,int)) );
    
    // connecting origin things for "Scripts", too
    connect( scripts, SIGNAL(sigOrigin(bool)), startRL[i], SLOT(setOrigin(bool)));
    connect( scripts, SIGNAL(sigOrigin(bool)), stopRL[i], SLOT(setOrigin(bool)));
    connect( startRL[i], SIGNAL(sigOrigin(bool)), scripts, SLOT(setOrigin(bool)));
    connect( stopRL[i], SIGNAL(sigOrigin(bool)), scripts, SLOT(setOrigin(bool)));
    
    // menus
    connect( startRL[i], SIGNAL(sigPopupMenu(KSVDragData*, QPoint, bool)), this, SLOT(slotPopupRLMenu(KSVDragData*, QPoint, bool)) );
    connect( stopRL[i], SIGNAL(sigPopupMenu(KSVDragData*, QPoint, bool)), this, SLOT(slotPopupRLMenu(KSVDragData*, QPoint, bool)));
    
    // doubleclick
    connect( startRL[i], SIGNAL(sigDoubleClick(KSVDragData*, QPoint)), this, SLOT(slotDoubleClick(KSVDragData*, QPoint)));
    connect( stopRL[i], SIGNAL(sigDoubleClick(KSVDragData*, QPoint)), this, SLOT(slotDoubleClick(KSVDragData*, QPoint)));
    
    // copy
    connect( startRL[i], SIGNAL(sigCopy(KSVDragData*)), this, SLOT(slotCopy(KSVDragData*)));
    connect( stopRL[i], SIGNAL(sigCopy(KSVDragData*)), this, SLOT(slotCopy(KSVDragData*)));
    
    // move
    connect( startRL[i], SIGNAL(sigMove(KSVDragData*)), this, SLOT(slotMove(KSVDragData*)));
    connect( stopRL[i], SIGNAL(sigMove(KSVDragData*)), this, SLOT(slotMove(KSVDragData*)));   
    
    // something changed
    connect( startRL[i], SIGNAL(sigChanged()), this, SLOT(slotChanged()));
    connect( stopRL[i], SIGNAL(sigChanged()), this, SLOT(slotChanged()));
    
    // use this loop for setting tooltips
    QToolTip::add(startRL[i], QString().sprintf(i18n("Drag here to start services\nwhen entering Runlevel %i"), i).data());
    QToolTip::add(stopRL[i], QString().sprintf(i18n("Drag here to stop services\nwhen entering Runlevel %i"), i).data());
  }
}

void KSysV::initScripts() {
  QDir scriptDir = QDir(pathToScripts);
  if (!scriptDir.exists())
	return;

  scriptDir.setFilter( QDir::Files | QDir::Hidden | QDir::NoSymLinks | QDir::Executable);        
  scriptDir.setSorting( QDir::Name );
  
  const QFileInfoList *scriptList = scriptDir.entryInfoList();        
  QFileInfoListIterator it( *scriptList ); 			                             // create list iterator        
  QFileInfo* fi; 							                     // pointer for traversing         
  while ( (fi=it.current()) ) {           			                             // for each file...            
    scripts->insertItem(fi->baseName().data(), "ksysv_exec.xpm", (char*) (fi->filePath()));
    ++it;        						                             // goto next list element       

    // keep GUI alive
    qApp->processEvents();
    
  }
}

void KSysV::initRunlevels() {
  for (int i = 0; i < 7; ++i) {
    QString _path = QString(pathToRLs.data());
    _path += QString().sprintf("/rc%i.d", i);
    if (!QDir(_path.data()).exists())
	continue;

    QDir d = QDir(_path.data());
    d.setFilter( QDir::Files );        
    d.setSorting( QDir::Name );
    
    const QFileInfoList *rlList = d.entryInfoList();        
    QFileInfoListIterator it( *rlList );	 			                     // create list iterator        
    QFileInfo* fi; 							                     // pointer for traversing         
    while ( (fi=it.current()) ) {           			                             // for each file...            
      info2Widget( fi, i);
      ++it;        						                             // goto next list element       

      
      // keep GUI alive
      qApp->processEvents();
      
    }
  }
}

void KSysV::info2Widget( QFileInfo* info, int index ) {
  QString f_link = QString(info->readLink().data());
  QString f_path = QString(info->filePath());
  QString f_dir = QString(info->dirPath(TRUE));
  QString f_name = QString(info->fileName().data());
  QString l_name = info->readLink();
  QString file;
  
  if ( f_link.isEmpty() )
    return;

  QFileInfo link_info = QFileInfo(l_name);
  QString l_base = QString(link_info.fileName().data());
  
  //  if (link_info.exists())
    file = relToAbs( f_dir , QString(link_info.filePath()) );

  // check if were in runlevel 6... (halt is reboot here)
  if ((index == 6) && (l_base == "halt"))
    l_base = "reboot";
  
  if ( f_name.left(1) == "S" )
    startRL[index]->insertItem( l_base.data(), "ksysv_start.xpm", file.data() );
  else 
    stopRL[index]->insertItem( l_base.data(), "ksysv_stop.xpm", file.data() );
}

void KSysV::slotWriteSysV() {
  for (int i = 0; i < 7; ++i) {
    clearRL(i);

    int maxIndex;

    startRL[i]->toFirst();
    maxIndex = startRL[i]->currentNrOfItems();
    for (int j = 0; j < maxIndex; ++j) {
      KSVDragData* cur_w = startRL[i]->current();
      writeToDisk(cur_w, i, j, TRUE);
      startRL[i]->next();
    }
    
    stopRL[i]->toFirst();
    maxIndex = stopRL[i]->currentNrOfItems();
    for (int j = 0; j < maxIndex; ++j) {
      KSVDragData* cur_w = stopRL[i]->current();
      writeToDisk(cur_w, i, j, FALSE);
      stopRL[i]->next();
    }
    
  }  
}

void KSysV::writeToDisk(KSVDragData* _w, int _rl, int _index, bool _start) {
  QString Number = QString();
  QString FirstLetter = QString();
  QString LastWord = _w->getTitle();

  if (_start)
    FirstLetter = "S";
  else
    FirstLetter = "K";
  if (_index < 10)
    Number.sprintf("0%i", _index);
  else
    Number.sprintf("%i", _index); 
  
  QString path = QString( pathToRLs.data() );
  path += QString().sprintf("/rc%i.d/", _rl);
  if ((_rl == 6) && (LastWord == "halt"))
    LastWord = QString("reboot");
  path += QString().sprintf("%s%s%s", FirstLetter.data(), Number.data(), LastWord.data());
  
  int err = symlink(_w->getFileName().data(), path.data());
  if (err != 0) {
	QString _txt = QString().sprintf(i18n("\nERROR: %i\nwhile making symlink\n%s%s%s\npointing to\n%s"), 
					 err, FirstLetter.data(), Number.data(), _w->getTitle().data(),
					 _w->getFileName().data());
	// msg(_txt);
	QString newText = textDisplay->text();
	newText += _txt;
	textDisplay->setText(newText.data());
	textDisplay->setCursorPosition(textDisplay->numLines(), 0, FALSE);	
  }
}

void KSysV::clearRL(int _rl) {
  QString _path = QString(pathToRLs.data());
  _path += QString().sprintf("/rc%i.d", _rl);
  
  QDir d = QDir(_path.data());
  d.setFilter( QDir::Files );        
  d.setSorting( QDir::Name );
  
  const QFileInfoList *rlList = d.entryInfoList();        
  QFileInfoListIterator it( *rlList );	 			// create list iterator        
  QFileInfo* fi; 						// pointer for traversing         
  while ( (fi=it.current()) ) {           			// for each file...         
    removeFile( fi, d );
    ++it;        						// goto next list element       
  }
}

void KSysV::removeFile( QFileInfo* _info, QDir _dir ) {
  if (!_dir.remove(_info->fileName(), FALSE))
    msg(QString().sprintf(i18n("Failed to remove\n%s\nPlease check Permissions."), _info->fileName().data()));
}

void KSysV::reInit() {

  // keep GUI alive
  qApp->processEvents();  

  for (int i = 0; i < 7; ++i) {    
    startRL[i]->clear();
    stopRL[i]->clear();
  }

  scripts->clear();
  initRunlevels();
  initScripts();
}

QString KSysV::getAppName() {
  QString _name = kapp->getCaption();
  if (_name == "ksysv")
    _name = QString("System V Init Editor");
  return _name;
}

void KSysV::setPathToScripts( QString _path ) {
  pathToScripts = QString( _path.data() );
  //	reInit();
}

void KSysV::setPathToRLs( QString _path ) {
  pathToRLs = QString( _path.data() );
  //	reInit();
}

void KSysV::slotChanged() {
  emit sigChanged();
}

void KSysV::msg(QString _text) {
  QMessageBox::warning(kapp->topWidget(), i18n("Warning"), _text.data());     
}

QString KSysV::relToAbs( QString _dir, QString _rel ) {  
  QString _abs;

  if (_rel.left(1) != "/") 
    _abs = QDir::cleanDirPath( (_dir + "/").data() + _rel);
  else
    _abs = QDir::cleanDirPath( _rel );
  return _abs;
}

void KSysV::slotPopupRLMenu(KSVDragData* _w, QPoint _pos, bool) {
  // normal slot
  if (_menu) {
    delete _menu;
    _menu = 0;
  }
  
  _currentW = _w;
  _menu = new QPopupMenu();
  //  _menu->insertItem(kil->loadIcon("ksysv_info.xpm"), i18n("&Info..."), this, SLOT(infoOnWidget()));

  // edit menu
  // _menu->insertSeparator();
  _menu->insertItem(i18n("C&ut"), this, SLOT(removeItem()));
  _menu->insertItem(i18n("&Copy"), this, SLOT(copyMenu()));
  _menu->insertItem(i18n("&Paste"), this, SLOT(removeItem()));
  // disable 
  _menu->setId(0,0);
  _menu->setId(1,1);
  _menu->setId(2,2);
  _menu->setItemEnabled(0, FALSE);
//   _menu->setItemEnabled(1, FALSE);
  _menu->setItemEnabled(2, FALSE);
  
  // normal remove
  _menu->insertSeparator();
  _menu->insertItem(i18n("&Remove"), this, SLOT(removeItem()));

  // properties
  _menu->insertSeparator();
  _menu->insertItem(i18n("&Properties..."), this, SLOT(infoOnWidget()));
  
  _menu->popup(_pos);
}

void KSysV::slotPopupScriptsMenu(KSVDragData* _w, QPoint _pos, bool ) {
  if (_menu) {
    delete _menu;
    _menu = 0;
  }
  _currentW = _w;
  _menu = new QPopupMenu();
  _menu->insertItem(kil->loadIcon("ksysv_info.xpm"), "&Info...", this, SLOT(infoOnWidget()));
  _menu->insertSeparator();
  _menu->insertItem(
		    // kil->loadIcon("ksysv_start.xpm"),
		    i18n("&Start"), this, SLOT(startW()));
  _menu->insertItem(
		    // kil->loadIcon("ksysv_stop.xpm"),
		    i18n("St&op"), this, SLOT(stopW()));
  
  _menu->insertSeparator();
  _menu->insertItem(
		    kil->loadIcon("ksysv_restart.xpm"), 
		    i18n("&Restart"), this, SLOT(restartW()));
  _menu->setId(0, 0);
  _menu->setId(2, 2);
  //  _menu->setItemEnabled(0, FALSE);
  //  _menu->setItemEnabled(2, FALSE);
  _menu->popup(_pos);
}

void KSysV::infoOnWidget() {
  QString msgStr = QString();
  if (_currentW) {
    msgStr.sprintf(i18n("%s\n\n%s is a link to\n%s"), _currentW->getRunlevel().data(), _currentW->getTitle().data(), _currentW->getFileName().data());
    QMessageBox::information(kapp->topWidget(), QString().sprintf(i18n("Properties of %s"), _currentW->getTitle().data()).data(), msgStr.data(), i18n("OK"));
    _currentW = 0; 
  }
}

void KSysV::removeItem() {
  if (_currentW) {
    QString name2seek = _currentW->getRunlevel();
    KSVWidgetList* parentList = 0;
    for (int i = 0; i < 7; ++i) {
      if (startRL[i]->name() == name2seek)
	parentList = startRL[i];
      else if (stopRL[i]->name() == name2seek)
	parentList = stopRL[i];
      if (parentList != 0)
	parentList->removeItem(_currentW);
    }
    _currentW = 0;
  }
}

void KSysV::stopW() {
  if (_currentW) {
    KProcess *_proc = new KProcess();
    
    _proc->clearArguments();

    *_proc << _currentW->getFileName().data() << "stop";

    connect(_proc, SIGNAL(processExited(KProcess*)), this, SLOT(slotExitedProcess(KProcess*)));
    connect(_proc, SIGNAL(receivedStdout(KProcess*, char*, int)), this, SLOT(slotOutput(KProcess*, char*, int)));
    connect(_proc, SIGNAL(receivedStderr(KProcess*, char*, int)), this, SLOT(slotErr(KProcess*, char*, int)));

    // refresh textDisplay
    QString newText = textDisplay->text();
    newText += QString().sprintf("** Stopping %s **\n", _currentW->getTitle().data());
    textDisplay->setText(newText.data());
    
    _proc->start(KProcess::NotifyOnExit, KProcess::AllOutput);
    
    // notify parent
    emit sigRun(_currentW->getFileName() + " stop");

    _currentW = 0;
  }
}

void KSysV::startW() {
  if (_currentW) {
    KProcess* _proc = new KProcess();
    
    _proc->clearArguments();
    
    *_proc << _currentW->getFileName().data() << "start";
    
    connect(_proc, SIGNAL(processExited(KProcess*)), this, SLOT(slotExitedProcess(KProcess*)));
    connect(_proc, SIGNAL(receivedStdout(KProcess*, char*, int)), this, SLOT(slotOutput(KProcess*, char*, int)));
    connect(_proc, SIGNAL(receivedStderr(KProcess*, char*, int)), this, SLOT(slotErr(KProcess*, char*, int)));
    
    // refresh textDisplay
    QString newText = textDisplay->text();
    newText += QString().sprintf("** Starting %s **\n", _currentW->getTitle().data());
    textDisplay->setText(newText.data());

    _proc->start(KProcess::NotifyOnExit, KProcess::AllOutput);
    
    // notify parent
    emit sigRun(_currentW->getFileName() + " start");
    
    _currentW = 0;
  }
}

void KSysV::restartW() {
  if (_currentW) {

    // stopping

    KProcess *_proc = new KProcess();
    
    _proc->clearArguments();
       
    *_proc << _currentW->getFileName().data() << "stop";

    connect(_proc, SIGNAL(processExited(KProcess*)), this, SLOT(slotExitDuringRestart(KProcess*)));
    connect(_proc, SIGNAL(receivedStdout(KProcess*, char*, int)), this, SLOT(slotOutput(KProcess*, char*, int)));
    connect(_proc, SIGNAL(receivedStderr(KProcess*, char*, int)), this, SLOT(slotErr(KProcess*, char*, int)));

    // refresh textDisplay
    QString newText = textDisplay->text();
    newText += QString().sprintf("** Stopping %s **\n", _currentW->getTitle().data());
    textDisplay->setText(newText.data());

    _proc->start(KProcess::NotifyOnExit, KProcess::AllOutput);

    // notify parent
    emit sigRun(_currentW->getFileName() + " stop");
  }
}

void KSysV::slotOutput( KProcess*, char* _buffer, int _buflen) {
  if (_buflen > 0) {
    QString buffer = QString( _buffer, _buflen + 1 ); 
    QString newText = textDisplay->text();
    newText += buffer;
    textDisplay->setText(newText.data());
    textDisplay->setCursorPosition(textDisplay->numLines(), 0, FALSE);
  }
}

void KSysV::slotErr( KProcess*, char* _buffer, int _buflen) {
  if (_buflen > 0) {
    QString buffer = QString( _buffer, _buflen + 1);
    QString newText = textDisplay->text();
    newText += buffer;
    textDisplay->setText(newText.data());
    textDisplay->setCursorPosition(textDisplay->numLines(), 0, FALSE);
  }
}

void KSysV::slotExitedProcess( KProcess* proc ) {
  QString newText = textDisplay->text();
  newText += "---------------------\n";
  textDisplay->setText(newText.data());
  textDisplay->setCursorPosition(textDisplay->numLines(), 0, FALSE);
  
  emit sigStop();
  delete proc;
  proc = 0;
}

void KSysV::slotToggleOut() {
  showOut = !showOut;
  if (!showOut) {
    textDisplay->hide();
    realHeight -= (textDisplay->height() + 4);
  } else {
    textDisplay->show();
    realHeight += (textDisplay->height() + 4);
  }
  setFixedSize(width(), realHeight);
  emit sigUpdateParent();
}

void KSysV::slotScriptsNotRemovable() {
  emit sigNotRemovable();
}

void KSysV::slotDoubleClick( KSVDragData* _data, QPoint) {
  _currentW = _data;
  infoOnWidget();
}

void KSysV::slotCopy( KSVDragData* _data ) {
  KSVCopyDialog* foo = new KSVCopyDialog(kapp->topWidget(), i18n("Copy"));
  int result = foo->exec();
  if (result) {
    int i = getRL(foo);

    // make new DataObject
    KSVDragData* newO = new KSVDragData();
    newO->fromString(_data->toString());    
    //    QString ss;
    if (foo->startBt->isChecked()) {
      // ss = "START";
      startRL[i]->insertItem(newO, 0);
      startRL[i]->repaint(FALSE);
    } else {
      //      ss = "STOP";
      stopRL[i]->insertItem(newO, 0);
      stopRL[i]->repaint(FALSE);
    }

    
  }
}

void KSysV::slotMove( KSVDragData* _data ) {
  KSVCopyDialog* foo = new KSVCopyDialog(kapp->topWidget(), i18n("Move"));
  int result = foo->exec();
  if (result) {
    int i = getRL(foo);
    
    // make new DataObject
    KSVDragData* newO = new KSVDragData();
    newO->fromString(_data->toString());    

    // and store a copy (only necessary in "move")
    KSVDragData* backupO = new KSVDragData();
    backupO->fromString(_data->toString());

    if (foo->startBt->isChecked()) {
      startRL[i]->insertItem(newO, 0);
      // remove only if it's not the same runlevel
      if (startRL[i]->name() != backupO->getRunlevel()) {
	for (int j = 0; j < 7; ++j) {
	  startRL[j]->slotRemoveAfterDrop(_data->getRunlevel(), _data);
	  stopRL[j]->slotRemoveAfterDrop(_data->getRunlevel(), _data);
	}
      }
      startRL[i]->repaint(FALSE);

    } else {
      stopRL[i]->insertItem(newO, 0);
      // remove only if it's not the same runlevel
      if (stopRL[i]->name() != backupO->getRunlevel()) {
	for (int j = 0; j < 7; ++j) {
	  stopRL[j]->slotRemoveAfterDrop(_data->getRunlevel(), _data);
	  startRL[j]->slotRemoveAfterDrop(_data->getRunlevel(), _data);
	}
      }
      stopRL[i]->repaint(FALSE);
    }
    
    // remove the backup object
    if (backupO) {
      delete backupO;
      backupO = 0;
    }
  }
}

void KSysV::slotExitDuringRestart( KProcess* proc ) {
  QString newText = textDisplay->text();
  newText += "---------------------\n";
  textDisplay->setText(newText.data());
  textDisplay->setCursorPosition(textDisplay->numLines(), 0, FALSE);

  delete proc;
  proc = new KProcess();
  
  proc->clearArguments();

  *proc << _currentW->getFileName().data() << "start";

  connect(proc, SIGNAL(processExited(KProcess*)), this, SLOT(slotExitedProcess(KProcess*)));
  connect(proc, SIGNAL(receivedStdout(KProcess*, char*, int)), this, SLOT(slotOutput(KProcess*, char*, int)));
  connect(proc, SIGNAL(receivedStderr(KProcess*, char*, int)), this, SLOT(slotErr(KProcess*, char*, int)));
  
  // refresh textDisplay
  newText = textDisplay->text();
  newText += QString().sprintf("** Starting %s **\n", _currentW->getTitle().data());
  textDisplay->setText(newText.data());
  
  proc->start(KProcess::NotifyOnExit, KProcess::AllOutput);
  
  // notify parent
  emit sigRun(_currentW->getFileName() + " start");
  
  _currentW = 0;
}

void KSysV::copyMenu() {
  if (_currentW) {
    slotCopy(_currentW);
    _currentW = 0;
  }
}

int KSysV::getRL(KSVCopyDialog* _dialog) {
  int result = 0;

  for (int i = 0; i < 7; ++i) {
    if (_dialog->rlBt[i]->isChecked())
      result = i;
  }
  
  return result;
}
