/*

    Requires the Qt widget libraries, available at no cost at
    http://www.troll.no

    Copyright (C) 1997 Peter Putzer
                       pputzer@edu.uni-klu.ac.at

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/     

#ifndef KSYSV_H
#define KSYSV_H

#include "trash.h"
#include <qfileinf.h>
#include <qwidget.h>
#include <qpopmenu.h>
#include <qmlined.h>
#include <kprocess.h>
#include <kiconloader.h>
#include "new_dwlst.h"
#include "copydialog.h"
#include "ksv_dragdata.h"

class KSysV : public QWidget 
{
  Q_OBJECT

 public:
  KSysV(QWidget* parent = 0, const char* name = 0);
  ~KSysV();

 protected:
  friend class KSVTopLevel;
  KSVWidgetList* startRL[7];
  KSVWidgetList* stopRL[7];
  KSVWidgetList* scripts;
  KSVTrash* trash;
  QString pathToScripts;
  QString pathToRLs;
  QPopupMenu* _menu;
  KSVDragData* _currentW;
  QMultiLineEdit* textDisplay;
  KIconLoader* kil;

  void initLList();
  void initScripts();
  void initRunlevels();
  void info2Widget( QFileInfo* info, int index );
  void writeToDisk( KSVDragData* _w, int _rl,int _index, bool _start );
  void clearRL( int _rl );
  static void removeFile( QFileInfo* info, QDir _dir );
  static QString relToAbs( QString _dir, QString _rel );

 public slots:
  void slotDropAction( KDNDDropZone* _zone );
  void slotWriteSysV();
  void reInit();
  void setPathToScripts( QString _path );
  void setPathToRLs( QString _path );
       
 protected slots:
  void slotChanged();
  void slotPopupRLMenu(KSVDragData* _w, QPoint _pos, bool _removable);
  void slotPopupScriptsMenu(KSVDragData* _w, QPoint _pos, bool _removable);
  void infoOnWidget();
  void removeItem();
  void startW();
  void stopW();
  void restartW();
  void slotOutput( KProcess* _p, char* _buffer, int _buflen );
  void slotErr( KProcess* _p, char* _buffer, int _buflen );
  void slotExitedProcess(KProcess* proc);
  void slotToggleOut();
  void slotScriptsNotRemovable();
  void slotDoubleClick( KSVDragData* _data, QPoint _pos );
  void slotCopy( KSVDragData* _data );
  void slotMove( KSVDragData* _data );
  void slotExitDuringRestart(KProcess* proc);
  void copyMenu();

 private:
  int realHeight;
  bool showOut;

 signals:
  void sigChanged();  
  void sigUpdateParent();
  void sigRun(QString _name);
  void sigStop();
  void sigNotRemovable();

 /* 
     static members (utility functions)
 */
 public:
  static QString getAppName();
  static void msg(QString _text);
  static int getRL(KSVCopyDialog* _dialog);
};

#endif
