/*

    Requires the Qt widget libraries, available at no cost at
    http://www.troll.no

    Copyright (C) 1997 Peter Putzer
                       pputzer@edu.uni-klu.ac.at

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/     

/*
    The pidfile portions of this code have been shamelessly stolen from kdehelp.
    These parts are (c) 1997 Martin Jones
*/

/****************************************************************
**
** kSysV
**
****************************************************************/

#include <qmsgbox.h>
#include <qlabel.h>
#include "ksvtop.h"
#include <kapp.h>
#include <kiconloader.h>
#include <signal.h>
#include <sys/stat.h>

#include "ksysv_ipc.h"
#include "error.h"
#include <kwm.h>
#include <time.h>
#include "ksysv.h"

static int msgqid = -1;
static QString pidFile;

// remove pid file

static void cleanup( int sig ) {
  printf( "cleanup: signal = %d\n", sig );
  if (msgqid >= 0)
    msgctl( msgqid, IPC_RMID, 0 );
  remove( pidFile );
  exit(0);
}        

// make sure the pid file is cleaned up when exiting unexpectedly.

void catchSignals() {
  //      signal(SIGHUP, cleanup);                /* Hangup */
  signal(SIGINT, cleanup);                /* Interrupt */
  signal(SIGTERM, cleanup);               /* Terminate */
  //      signal(SIGCHLD, cleanup);
  
  signal(SIGABRT, cleanup);
  signal(SIGALRM, cleanup);
  signal(SIGFPE, cleanup);
  signal(SIGILL, cleanup);
  signal(SIGPIPE, cleanup);
  signal(SIGQUIT, cleanup);
  //      signal(SIGSEGV, cleanup);
  
#ifdef SIGBUS
  signal(SIGBUS, cleanup);
#endif
#ifdef SIGPOLL
  signal(SIGPOLL, cleanup);
#endif
#ifdef SIGSYS
  signal(SIGSYS, cleanup);
#endif
#ifdef SIGTRAP
  signal(SIGTRAP, cleanup);
#endif
#ifdef SIGVTALRM
  signal(SIGVTALRM, cleanup);
#endif
#ifdef SIGXCPU
  signal(SIGXCPU, cleanup);
#endif
#ifdef SIGXFSZ
  signal(SIGXFSZ, cleanup);
#endif
}                


//  timer used to monitor msg queue for request for new window

class Timer : public QObject {
public:
  Timer() {
    startTimer( 100 );
  }
  
protected:
  virtual void timerEvent( QTimerEvent * );
};

void Timer::timerEvent( QTimerEvent * ) {
  KSysVMsg buf, retbuf;
  
  if ( buf.recv( msgqid, 3L ) != -1 ) {
    retbuf.setType( 4L );
    retbuf.send( msgqid );
     
    KWM::activate(kapp->mainWidget()->winId());
  }    
}

int main( int argc, char **argv ) {
  FILE *fp;
  
  // so that everything is cleaned up
  catchSignals();
  
  Timer *timer = new Timer;
                                             
  KApplication app( argc, argv );

  // display splash screen during loading
  KIconLoader* kil = new KIconLoader();
  QLabel* splash = new QLabel(0, "Splash", WStyle_Customize | WStyle_NoBorder | WStyle_Tool);
  splash->setPixmap(kil->loadIcon("ksysv_splash.xpm"));
  splash->setFrameStyle( QFrame::WinPanel | QFrame::Raised );
  splash->resize(splash->sizeHint());
  splash->show();  

  // create data directory if necessary
  QString p = getenv( "HOME" );
  QString rcDir = p + "/.kde/share/apps";
  if ( access( rcDir, F_OK ) )
    mkdir( rcDir, 0740 );
  
  rcDir += "/ksysv";
  if ( access( rcDir, F_OK ) )
    mkdir( rcDir, 0740 );
  
  pidFile = rcDir + "/ksysv.pid";
  
  // if there is a pidFile then this is not the first instance of ksysv
  if ( ( fp = fopen( pidFile, "r" ) ) != NULL ) {
    KSysVMsg buf;
    int pid;
    QString msg = "";
    buf.setType( 3L );
    buf.setMsg( msg );
    fscanf( fp, "%d %d", &pid, &msgqid );
    // if this fails I assume that the pid file is left over from bad exit
    // and continue on
    
    if ( buf.send( msgqid ) != -1) {
      // if we don't receive a reply within 3 secs assume previous
      // instance of ksysv died an unnatural death.
      // How should this stuff be handled properly?
      
      time_t start = time(NULL);
      while ( time(NULL) - start < 3 ) {
	if (buf.recv( msgqid, 4L, IPC_NOWAIT ) != -1) {
	  fclose( fp );
	  exit(0);
	}
      }
      msgctl( msgqid, IPC_RMID, NULL );
    }
    fclose( fp );
  }

  // This is the first instance so create a pid/msgqid file
  key_t key = ftok( getenv( "HOME" ), (char)rand() );
  msgqid = msgget( key, IPC_CREAT | 0600 );
  
  fp = fopen( pidFile, "w" );
  fprintf( fp, "%ld %d\n", (long)getpid(), msgqid );
  fclose( fp );                                    
  
  // session-management
  KSVTopLevel* top = new KSVTopLevel();

  if (kapp->isRestored() && KTopLevelWidget::canBeRestored(1)) {
    top->restore(1);
    kapp->setMainWidget( top );
  } else {
    kapp->setMainWidget( top );
    top->setCaption(KSysV::getAppName());
    top->show();
  }
  // end session-management
  
  splash->hide();
  app.exec();
  
  // clean up
  delete kil;
  delete splash;
  delete timer;
  msgctl( msgqid, IPC_RMID, 0 );  
  remove( pidFile );   
}
