/*

    Requires the Qt widget libraries, available at no cost at
    http://www.troll.no

    Copyright (C) 1997 Peter Putzer
                       pputzer@edu.uni-klu.ac.at

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/     

#include "new_dwlst.h"
#include <kapp.h>

// include meta-object code
#include "new_dwlst.moc"

// define constants
#define ENTRY_HEIGHT 19
#define VISIBILITY_OFFSET 0

KSVWidgetList::KSVWidgetList( QWidget* parent, char* name ) : QFrame(parent, name) {
  // set Panel-Style
  if (kapp->style() == WindowsStyle)
    setFrameStyle( WinPanel | Sunken );
  else
    setFrameStyle( Panel | Sunken );        
  setLineWidth(1);

  // initialize scrollbar
  bar = new QScrollBar(0, 0, 0, 0, 0, QScrollBar::Vertical, this, "scrollbar");
  connect(bar, SIGNAL(valueChanged(int)), this, SLOT(slotValueChanged(int)));

  // initialize class members
  maxVisible = 10;
  borderWidth = 2;
  widgetWidth = 80;
  curIndex = 0;
  offset = 0;
  removable = TRUE;
  dropTarget = TRUE;
  defaultIcon = "unknown.xpm";
  viewPort = new QWidget(this, "VIEWPORT");
  viewPort->setBackgroundMode( PaletteBase );
  KDNDDropZone* zone = new KDNDDropZone(viewPort, DndText);
  connect(zone, SIGNAL(dropAction( KDNDDropZone* )), this, SLOT(slotRootDrop(KDNDDropZone*)));

  // dummy DropZone for preventing errors...
  KDNDDropZone* dummy_zone = new KDNDDropZone(this, DndText);
  dummy_zone->getAcceptType(); // to make the compiler happy...

  oldSliderValue = 0;
  
  // initialize "slider"
  movingPart = new KSVMovingPart(viewPort, "MOVINGPART");
  movingPart->setGeometry(0,0,80,800);

  // initialize canvas
  canvas = new KSVDragList(movingPart, "CANVAS");
  canvas->setRunlevel(QString(QWidget::name())); // use the widget name as "runlevel"
  //  connect(canvas, SIGNAL(sigDropOccured(KDNDDropZone*, KSVDragList*, bool)), this, SLOT(updateBar()));
  connect(canvas, SIGNAL(sigResize(int, int)), movingPart, SLOT(slotResize(int, int)));
  connect(canvas, SIGNAL(sigRightPressed(KSVDragList*, KSVDragData*, QPoint)), this, SLOT(slotPopMenu(KSVDragList*, KSVDragData*, QPoint)));
  connect(canvas, SIGNAL(sigChanged()), this, SLOT(updateBar()));
  connect(canvas, SIGNAL(sigScrollUp(int)), this, SLOT(keyboardUp(int)));
  connect(canvas, SIGNAL(sigScrollDown(int)), this, SLOT(keyboardDown(int)));
  connect(canvas, SIGNAL(sigDroppedFrom(QString, KSVDragData*)), this, SLOT(slotDroppedFrom(QString, KSVDragData*)));
  connect(canvas, SIGNAL(sigNotRemovable()), this, SLOT(slotNotRemovable()));
  connect(canvas, SIGNAL(sigDoubleClick(KSVDragList*, KSVDragData*, QPoint)), this, SLOT(doubleClickOnEntry(KSVDragList*, KSVDragData*, QPoint)));
  connect(canvas, SIGNAL(sigCopy(KSVDragList*, KSVDragData*)), this, SLOT(slotCopy(KSVDragList*, KSVDragData*)));
  connect(canvas, SIGNAL(sigMove(KSVDragList*, KSVDragData*)), this, SLOT(slotMove(KSVDragList*, KSVDragData*)));
  
  // capture mouse moves
  connect(canvas, SIGNAL(mouseMove(int,int)), this, SLOT(slotMouseMove(int,int)));

  // connect origin sigs & slots
  connect(canvas, SIGNAL(sigOrigin(bool)), this, SLOT(slotOrigin(bool)));

  // set focusPolicy
  viewPort->setFocusProxy(canvas);
  viewPort->setFocusPolicy(ClickFocus);
  // I get a strange graphics error when I do not comment out the following statements
  //   bar->setFocusProxy(canvas); 
  //   bar->setFocusPolicy(ClickFocus);
  this->setFocusProxy(canvas);
  this->setFocusPolicy(StrongFocus);
  
  // adjust size 
  adjustSize();
}

KSVWidgetList::~KSVWidgetList() {
  // nothing yet
}

void KSVWidgetList::adjustSize() {
  viewPort->setGeometry( borderWidth, borderWidth, 80 - 1, ENTRY_HEIGHT * maxVisible + 1);
  movingPart->slotResize(canvas->width(), canvas->height());
  bar->setGeometry( 100 - borderWidth - 17, borderWidth, 17, ENTRY_HEIGHT * maxVisible);
  resize(100, ENTRY_HEIGHT * maxVisible + 1 + 2*borderWidth);
}


void KSVWidgetList::slotPopMenu(KSVDragList* _w, KSVDragData* _data, QPoint _pos) {
//   _pos -= QPoint(10,0);
  _pos = _w->mapToGlobal(_pos);
  emit sigPopupMenu(_data, _pos, removable);
}


void KSVWidgetList::setMaxVisible(int _max) {
  maxVisible = _max;
  adjustSize();
  //update();
}

void KSVWidgetList::updateBar() {
  if ( canvas->count() > maxVisible) {
    bar->setRange(0, (canvas->count() - maxVisible)*ENTRY_HEIGHT);
    bar->setSteps(ENTRY_HEIGHT, maxVisible*ENTRY_HEIGHT);
  } else {
    bar->setRange(0, 0);
    bar->setSteps(0, 0);
  }

  // inform the parent that some change has occured
  emit sigChanged();
}

void KSVWidgetList::setRemovable(bool _value) {
  removable = _value;
  canvas->setRemovable(removable);
}

void KSVWidgetList::setBorderWidth(int _value) {
  if (_value > 0)
    borderWidth = _value;
  else
    borderWidth = 0;
  //   updateContent();
  adjustSize();
}

bool KSVWidgetList::isRemovable() {
  return removable;
}

int KSVWidgetList::whatBorderWidth() {
  return borderWidth;
}

void KSVWidgetList::setDropTarget(bool _value) {
  dropTarget = _value;
}

bool KSVWidgetList::isDropTarget() {
  return dropTarget;
}


void KSVWidgetList::setDefaultIcon( QString _name) {
  defaultIcon = _name;
  canvas->setDefaultIcon(_name.data());
}

void KSVWidgetList::setWidth( int _value ) {
  if (_value > 0)
    widgetWidth = _value;
  else
    widgetWidth = 50;
//   updateContent();
  adjustSize();
}

void KSVWidgetList::clear() {
  // keep GUI alive
  qApp->processEvents();
    
  canvas->clear();
}

void KSVWidgetList::slotValueChanged( int _newValue ) {
  movingPart->scroll(0, oldSliderValue - _newValue);
  oldSliderValue = _newValue;
//   offset = (_newValue % ENTRY_HEIGHT) - maxVisible;
//   if (offset < 0)
//     offset = 0;
}

void KSVWidgetList::insertItem(char* _title, char* _icon, char* _file) {
  // used only in initializing... at least I hope so
  QString _data;

  _data.sprintf("%s|%s|%s|%s", _title, _icon, _file, "RunlevelX");
  KSVDragData* foo = new KSVDragData();
  foo->fromString(_data);
  canvas->addItem(canvas->count(), foo);
}

void KSVWidgetList::insertItem(KSVDragData* _w, int _pos) {
  canvas->addItem(_pos, _w);
}

void KSVWidgetList::removeItem(KSVDragData* _w) {
  canvas->removeItem(_w);
}

void KSVWidgetList::slotRootDrop( KDNDDropZone* _zone ) {
  QString droppedData = _zone->getData();
  if ( (_zone->getAcceptType() == _zone->getDataType()) ) {
    
    KSVDragData* newO = new KSVDragData();
    newO->fromString(droppedData);

    // for removing item elsewhere
    QString _old_rl = newO->getRunlevel();

    canvas->addItem( canvas->count(), newO );

    if (_old_rl != canvas->getRunlevel())
      emit sigDroppedFrom(_old_rl, newO);
//     emit sigDropOccured( _zone, this, FALSE );
  }
}

void KSVWidgetList::keyboardUp(int _focus) {
  if ( (_focus + 1 - VISIBILITY_OFFSET <= offset) && (offset > 0) ) {
    bar->setValue(bar->value() - ENTRY_HEIGHT);
    offset--;
  }
}

void KSVWidgetList::keyboardDown(int _focus) {
  if ( (_focus + VISIBILITY_OFFSET >= offset + maxVisible) && (offset + maxVisible < canvas->count()) ) {
    bar->setValue(bar->value() + ENTRY_HEIGHT);
    offset++;
  }
}

void KSVWidgetList::slotRemoveAfterDrop(QString _parent, KSVDragData* _o) {
  if ( (_parent == QString(name())) && isRemovable() )
    canvas->removeItem(_o);
}


void KSVWidgetList::slotDroppedFrom(QString _par, KSVDragData* _o) {
  emit sigDroppedFrom(_par, _o);
}

void KSVWidgetList::slotNotRemovable() {
  emit sigNotRemovable();
}

void KSVWidgetList::toFirst() {
  curIndex = 0;
}

void KSVWidgetList::next() {
  if (curIndex < canvas->count() - 1)
    curIndex++;
}

KSVDragData* KSVWidgetList::current() {
  return canvas->at(curIndex);
}

int KSVWidgetList::currentNrOfItems() {
  return canvas->count();
}

void KSVWidgetList::doubleClickOnEntry(KSVDragList*, KSVDragData* _data, QPoint _pos) {
  emit sigDoubleClick(_data,_pos);
}

void KSVWidgetList::slotCopy(KSVDragList*, KSVDragData* _data) {
  emit sigCopy(_data);
}

void KSVWidgetList::slotMove(KSVDragList*, KSVDragData* _data) {
  // only if data != 0
  if (_data)
    emit sigMove(_data);
}

void KSVWidgetList::autoScroll( int _x, int _y ) {
  QPoint globalPos = QPoint( _x, _y );
  QPoint localPos = parentWidget()->mapFromGlobal(globalPos);
  int lx = localPos.x();
  int ly = localPos.y();
  
  if ( (x() <= lx) && (lx <= (x() + width())) )
    if ( (y() <= ly) && (ly <= (y() + height())) ) {

      int downVal = y() + height() - ly;
      int upVal = ly - y();
      int scrollArea = (int)(1.5 * ENTRY_HEIGHT);
      int amount = ENTRY_HEIGHT;

      if ( (0 <= downVal) && (downVal <= scrollArea) ) {
  	// scroll down
  	bar->setValue(bar->value() + amount);
	if (offset < (canvas->count() - maxVisible))
	  offset++;  	
      } else if ( (0 <= upVal) && (upVal <= scrollArea) ) {
  	// scroll up
  	bar->setValue(bar->value() - amount);
	if (offset > 0)
	  offset--;
      }
    }
}

void KSVWidgetList::slotMouseMove( int _x, int _y ) {
  emit mouseMove( _x, _y);
}

void KSVWidgetList::slotOrigin( bool val ) {
  emit sigOrigin( val );
}

void KSVWidgetList::setOrigin( bool val ) {
  canvas->setOrigin(val);
  if (!val)
    canvas->repaint(FALSE);
}
