/*

    Requires the Qt widget libraries, available at no cost at
    http://www.troll.no

    Copyright (C) 1997 Peter Putzer
                       pputzer@edu.uni-klu.ac.at

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/     

#ifndef NEW_DWLST_H
#define NEW_DWLST_H

#include <qframe.h>
#include <qpopmenu.h>
#include <qscrbar.h>
#include <qstring.h>
#include "mylist.h"
#include "draglist.h"
#include "ksv_dragdata.h"
#include "moving.h"

/*
  KSVWidgetList: a scrolled list of draggable items.
*/	

class KSVWidgetList : public QFrame
{
  Q_OBJECT
 public:
  KSVWidgetList( QWidget* parent = 0, char* name = 0);
  ~KSVWidgetList();

 protected:
  int maxVisible;
  int curIndex;
  int borderWidth;
  int widgetWidth;
  bool removable;
  bool dropTarget;
  QString defaultIcon;
  QWidget* viewPort;
  QScrollBar* bar;
  KSVDragList* canvas;
  KSVMovingPart* movingPart;
  int offset;
  void adjustSize();
  // overidden members:

  
 public:
  bool isRemovable();
  bool isDropTarget();
  int  whatBorderWidth();
  int currentNrOfItems();
  void clear();
  void toFirst();
  void next();
  KSVDragData* current();
  
 public slots:
  void insertItem(char* _title, char* _icon, char* _file);
  void insertItem(KSVDragData* _w, int _pos);
  void removeItem(KSVDragData* _w);
  void setMaxVisible(int _max);
  void setRemovable(bool _value);
  void setBorderWidth(int _value);
  void slotRemoveAfterDrop(QString _parent, KSVDragData* _w);
  void setDropTarget(bool _value);
  void setDefaultIcon(QString _name);
  void setWidth(int _value);
  void setOrigin( bool val = TRUE );
            
 signals:
  void sigDroppedFrom(QString _par, KSVDragData* _w);
  void sigChanged();
  void sigPopupMenu(KSVDragData* _data, QPoint _pos, bool _removable);
  void sigNotRemovable();
  void sigDoubleClick(KSVDragData* _data, QPoint _pos);
  void sigCopy(KSVDragData* _data);
  void sigMove(KSVDragData* _data);
  void mouseMove( int _x, int _y );
  void sigOrigin( bool val );

 protected slots:
  void slotRootDrop(KDNDDropZone* _zone); 
  void slotPopMenu(KSVDragList* _w, KSVDragData* _data, QPoint _pos);
  void slotValueChanged(int _newValue);
  void updateBar();  
  void keyboardDown(int _focus);
  void keyboardUp(int _focus);
  void slotDroppedFrom(QString, KSVDragData*);
  void slotNotRemovable();
  void doubleClickOnEntry(KSVDragList* _list, KSVDragData* _data, QPoint _pos);
  void slotCopy(KSVDragList* _list, KSVDragData* _data);
  void slotMove(KSVDragList* _list, KSVDragData* _data);
  void autoScroll( int _x, int _y );
  void slotMouseMove( int _x, int _y );
  void slotOrigin( bool val );

 private:
  int oldSliderValue;
};

#endif
