/*

    Requires the Qt widget libraries, available at no cost at
    http://www.troll.no

    Copyright (C) 1997 Peter Putzer
                       pputzer@edu.uni-klu.ac.at

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/     

// KSysV Preferences Dialog

#include <qlabel.h>
#include <qpushbt.h>
#include <ktabctl.h>
#include "ksysv.h"
#include "prefs.h"
#include "typelayout.h"

// include meta-object code
#include "prefs.moc"


KSVPrefDlg::KSVPrefDlg( QWidget* parent = 0, char* name = 0) : QDialog(parent, name, TRUE)
{
//   int _y = 5;
  setCaption(KSysV::getAppName() + i18n(" - Preferences"));

  TLForm* f = new TLForm("root", caption(), this, this->parentWidget());
  KTypeLayout* l = f->layout;

  //  KTabCtl* my_book = (KTabCtl*) 
  l->addBook("book"); // ->widget;

  l->addPage("path_group", i18n("Path Configuration"));

  // separator
  l->addLabel("empty", "");
  l->newLine();

  // Script Path
  l->skip();
  QLabel* _prefix = (QLabel*) l->addLabel("prefix", i18n("Path to the directory containing your init-scripts (&services):"))->widget;
  l->skip();
  l->newLine();
  l->skip();
  _scriptPath = (QLineEdit*) l->addLineEdit("script_path", "", 100)->widget;
  l->skip();
  _prefix->setBuddy(_scriptPath);
  l->newLine();

  // separator
  l->addLabel("empty", "");
  l->newLine();

  // Runlevel Path
  l->skip();
  QLabel* _pRL = (QLabel*) l->addLabel("rl-prefix", i18n("Path to the directory containing your &runlevel-directories:"))->widget;
  l->skip();
  l->newLine();
  l->skip();
  _rlPath = (QLineEdit*) l->addLineEdit("rl_path", "", 100)->widget;
  l->skip();
  _pRL->setBuddy(_rlPath);
  l->newLine();

  // separator
  l->addLabel("empty", "");
  l->newLine();

  // end page
  l->endGroup(); // page "path"

/*
  // new Page "Links"
  l->addPage("link_group", i18n("Links"));
  
  l->newLine(); l->skip();
  l->addGroup("link_bg", i18n("Density"), TRUE);
  QRadioButton* soft_bt = (QRadioButton*) l->addRadioButton("1", "link_bg", i18n("&Soft"), TRUE, 0)->widget;
  l->newLine();
  QRadioButton* hard_bt = (QRadioButton*) l->addRadioButton("2", "link_bg", i18n("&Hard"), FALSE, 1)->widget;
  l->endGroup(); // link_bg
  l->skip();
  l->newLine(); l->skip();

  l->addGroup("link_rel", i18n("Relativity"), TRUE);
  QRadioButton* rel_bt = (QRadioButton*) l->addRadioButton("1,", "link_rel", i18n("&Relative"), FALSE, 0)->widget;
  l->newLine();
  QRadioButton* abs_bt = (QRadioButton*) l->addRadioButton("2", "link_rel", i18n("&Absolute"), TRUE, 1)->widget;
  l->endGroup(); // link_Rel
  l->skip();
  
  l->newLine();
  l->endGroup(); // Links
  // disable
  hard_bt->setEnabled(FALSE);
  rel_bt->setEnabled(FALSE);
  // ...and enable
  my_book->setTabEnabled("path_group", TRUE);
  
  l->setAlign("link_rel",AlignLeft);
  l->setAlign("link_bg", AlignLeft);
*/

  // end book
  l->endGroup(); // book
  l->newLine();

  // buttons
  l->addGroup ("buttons");
  l->skip();
  _ok = (QPushButton*) l->addButton("ok",i18n("OK"))->widget;
  _cancel = (QPushButton*) l->addButton("cancel",i18n("Cancel"))->widget;
  l->setAlign("ok",AlignRight|AlignVCenter);
  l->setAlign("cancel",AlignVCenter);
  l->endGroup(); // buttons
  
  //  l->endGroup();
  l->activate();
  
  // setup some stuff
  _ok->setAutoDefault(TRUE);
  _ok->setDefault(FALSE);
  _cancel->setAutoDefault(TRUE);
  _cancel->setDefault(TRUE);
  _cancel->setFocus();

  connect(_ok, SIGNAL(clicked()), this, SLOT(accept()));
  connect(_cancel, SIGNAL(clicked()), this, SLOT(reject()));

  // activate Layout Manager
  l->activate();

  // make fixed size
  setFixedSize(size());
}

// Destructor

KSVPrefDlg::~KSVPrefDlg() {
  
} 
